/*+--------------------------------------------------------------------------+
 *| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
 *| This software contains proprietary and confidential information of       |
 *| Rational and its suppliers. Use, disclosure or reproduction is           |
 *| prohibited without the prior express written consent of Rational.        |
 *+--------------------------------------------------------------------------+
 */
/*                         tms_arguments.h                   */

#ifndef TMS_ARGUMENTS_H
#define TMS_ARGUMENTS_H
#include "testmate/tms_header_macros.h"
TMS_BEGIN_CPLUSPLUS

/*
// It would be much nicer to have a communications record that simply
// has an entry for each call type.  Because most of the TMS routines
// use strings for parameters, passing multiple unconstrained values
// in a record that is supposed to handle several different program
// signatures becomes very unruly.  So, this package is the compromise.
//
// All signatures are encoded into a byte array in the following format:
//
//    <procedure-id> & <p1-size> & <p1> & <p2-size> & <p2> ...
//
// where p1 and p2 are parameter one and two respectively, the <pX-size>
// entries are a fixed two bytes in length, and any number of parameters
// can be added to the array.
//
// Encoding of a parameters signature is performed by concatenation of
// argument byte arrays by the caller.  Decoding is performed through
// an initial decode call followed by a sequence of calls to get the
// various paramters.  The decoder must know the order and the type of
// the parameters in an encoded byte array.
*/

enum Routine_Names {
    /* a null request; ignored */
    Rn_Tms_None = 0,
    /* Package TMS routines */
    Rn_Tms_Register_Result,
    Rn_Tms_Assert,
    Rn_Tms_Check_Assertions,
    Rn_Tms_Traverse_Assertions,
    Rn_Tms_Number_Of_Tests, 
    Rn_Tms_Prior_Test_Case_Result,
    Rn_Tms_Get_Test_Case_Result,
    Rn_Tms_Put_Line,
    Rn_Tms_Set_Log_Filter,
    Rn_Tms_Get_Switch,
    Rn_Tms_Register_Env,
    /* Script support routines */
    Rn_Comm_Defer_Results,
    Rn_Comm_Resolve_Placeholder,
    Rn_Comm_Test_List_Initialize,
    Rn_Comm_Start_Test_Case,
    Rn_Comm_Start_Driver,
    Rn_Comm_End_Driver,
    Rn_Comm_End_Test_Case,
    Rn_Comm_Coverage_Pre_Run,
    Rn_Comm_Coverage_Post_Run,
    Rn_Comm_Getenv
};


/*
// Values are encoded into the supplied To_String and the number of bytes
// inserted into the character array is returned as the result.  A negative
// result implies an errant encoding.
*/

/* extern int Encode_Routine_Name  TMS_PROTO((Routine_Names The_Routine, char *To_String)); */
extern int Encode_Routine_Name  TMS_PROTO((int The_Routine, char *To_String));

extern int Encode_String  TMS_PROTO((char *The_String, char *To_String));
extern int Encode_Integer  TMS_PROTO((int The_Integer, char *To_String));
extern int Encode_Enum  TMS_PROTO((int The_Position, char *To_String));


/*
// Values are decoded from the supplied From_String and the number of bytes
// checked in the supplied character array is returned as a result.
*/

/* extern int Decode_Routine_Name  TMS_PROTO((Routine_Names *The_Routine, char *From_String)); */
extern int Decode_Routine_Name  TMS_PROTO((int *The_Routine, char *From_String)); 


extern int Decode_String  TMS_PROTO((char *The_String, char *From_String));
extern int Decode_Integer  TMS_PROTO((int *The_Integer, char *From_String));
extern int Decode_Enum  TMS_PROTO((int *The_Position, char *From_String));

TMS_END_CPLUSPLUS

#endif

