# +--------------------------------------------------------------------------+
# | Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
# | This software contains proprietary and confidential information of       |
# | Rational and its suppliers. Use, disclosure or reproduction is           |
# | prohibited without the prior express written consent of Rational.        |
# +--------------------------------------------------------------------------+
#
###############################################################################
# Build Properties
#
# This file contains a set of "build properties" which define the meaning of
# build operations in a view.


###############################################################################
# Basic Properties
#
# These properties define the basic meaning of the build properties and
# are used to interpret the other properties.
# The host and target architectures may also be specified in the 
# Architecture.prop file although they are usually specified in the
# compiler home directory.

Build_Key_Version: 8.0

Build_Class: Rational_Build

Languages: C++ Java


###############################################################################
# Subclasses and Directory Content
#
# These properties define any extensions to the class hierarchy and also
# specify if automatic reaction is required when object of some classes
# are created or deleted.

{Subclasses:
{
  # <name>        <base-class> <compilation-state> <extensions>        <names>
  # string,         string,      string,           list,               list  
  
  { closure,        file,        source,  	   { .closure },       {} 	 },
  { disasm,         file,	 source,	   { .das },	       {}        },	

  { java,           file,        source,           { .java },          {}        },
  { java_class,     file,        source,           { .class },         {}        },
  { java_disasm,    file,	 source,	   { .jdas },	       {}        },
  
  { idl,	    file,        source,           { .idl },           {}        },

  { shared_library, file,        source,	   { .so, .sl },       {}        }  
}
Subclasses}

###############################################################################
# Directory Content
#
# These properties control how the system reacts to the creation and
# deletion of objects.

Subclasses.Directory_Content.File: ".Makefiles.rtnl/Directory_Content"

{Subclasses.Directory_Content.Classes_To_Notice:
    header source assembly directory java idl
Subclasses.Directory_Content.Classes_To_Notice}

{Subclasses.Directory_Content.Classes_To_Ignore:
    configuration saved object_file core closure disasm java_class java_disasm 
Subclasses.Directory_Content.Classes_To_Ignore}

{Subclasses.Directory_Content.Names_To_Ignore:
    lib*.a* lib*.so* lib*.sl* *.rtnl .* *~ #*# !*
Subclasses.Directory_Content.Names_To_Ignore}

###############################################################################
# Makefile Control
#
# These properties define the makefiles in a view.  Depending on the exact
# properties, these makefiles may be propagated from the build key into the
# view, and may have macros which are automatically updated by Apex.

{Makefile.Managed.View:
{
  { Definitions.mk, 	 .Rational/Build_Control, {},       {}, {} },
  { Master.mk,      	 .Rational/Build_Control, {},       {}, {} },
  { Includes.mk,    	 .Rational/Build_Control, {import}, {}, {} }
}
Makefile.Managed.View}

{Makefile.Managed.Directory:
{
  { Makefile,        "",              {static, switch}, {directory}, {} },
  { Context.mk,      .Makefiles.rtnl, {static, switch}, {directory}, {} },
  { Options_s.mk,    .Makefiles.rtnl, {switch},         {directory}, {} },
  { Options_c.mk,    .Makefiles.rtnl, {switch},         {directory}, {} },
  { Options_cpp.mk,  .Makefiles.rtnl, {switch},         {directory}, {} },
  { Options.mk,      .Makefiles.rtnl, {switch},         {directory}, {} },
  { Objects.mk,      .Makefiles.rtnl, {dynamic},        {directory}, {} },
  { Dependencies.mk, .Makefiles.rtnl, {dynamic},        {directory}, {} },
  { Query.mk,	     .Makefiles.rtnl, {static},         {directory}, {} }
}
Makefile.Managed.Directory}

{Makefile.Initialized.View:
{
  { Customization.mk,        Policy, 	      {switch},  {},          {} }
#  { Customization.prop,      Policy,          {},       {},          {} }
}
Makefile.Initialized.View}

Makefile.Initialized.Directory: {}

Makefile.Assisted.View: {}

Makefile.Assisted.Directory: {}

Makefile.Directory.Inclusions: *
Makefile.Directory.Exclusions: Templates* .lines .lines/* .nets .nets/* ptrepository cxx_repository
#
# Controls the directories which will contain makefiles.

{Makefile.Dependency_Mode: "<set,man_auto><if><view'switch(dependencies):b><then>auto<else>man<endif><endset>"
  			+ "<if><view'switch(dependencies_info_only):b><then>"
                        +     "<set,info_check,info>"
                        + "<else>"
                        +     "<set,info_check,check.del>"
                        + "<endif>"
                        + "<man_auto>.<info_check>"
Makefile.Dependency_Mode}                       
#
# Determines how dependencies will be generated and used.
#
# Possible values for Dependency_Mode:
#  none		    - No dependencies are generated
#
#  auto.info	    - Dependencies are automatically generated for information but are not used by makefiles
#  auto.check	    - Dependencies are automatically generated for information and makefile checks
#  auto.check.del   - Dependencies are automatically generated for information and makefile checks including
#		      graceful handling of deletion via dummy rules.
#  man.info	    - Dependencies are manually generated for information but are not used by makefiles
#  man.check	    - Dependencies are manually generated for information and makefile checks
#  man.check.del    - Dependencies are manually generated for information and makefile checks including
#		      graceful handling of deletion via dummy rules.


###############################################################################
# Makefile Macro Control
#
# These properties defined makefile macros whose values will be managed by
# Apex.  In general, the value of each macro is specified by a format string
# which may contain replacement symbols.

{Makefile.Macros.Static:
{
  { SUBSYSTEM,        "<subsystem>" 	 },
  { VIEW,             "<view>"      	 },
  { VIEW_SUBPATH,     "<view:s>"      	 },
  { DIRECTORY,        "<directory>" 	 },
  { DIRECTORY_CLASS,  "<directory'class>"},
  { KEY,              "<key>"       	 },
  { KEY_VERSION,      "<switch(build_key_version)>"},
  { UMASK,	      "<view'umask>"	 },
  { JAVA_PACKAGE,     "<if><dir><eq><view><then><subsystem:t:r><else><subsystem:t:r>.<dir:v:subst/.><endif>" },
  { MAKE_PROG,	      "<switch(Command_Translation.Make)>"}
}
Makefile.Macros.Static}

{Makefile.Macros.Dynamic:
{
  { SUBDIRS,         {directory},        {},         {},              {ignore},                 "{<object:t>, }"         },
  { C_SOURCES,       {c_source},         {},         {},              {ignore},               	"{<object:t>, }"         },
  { CPP_SOURCES,     {source},           {c_source}, {},              {ignore},               	"{<object:t>, }"         },
  { HEADERS,         {header},           {},         {},              {ignore},               	"{<object:t>, }"         },
  { S_OBJECTS,       {assembly},         {},         {},              {main_programs, ignore}, 	"{<object:t:r>.o, }" 	 },
  { C_OBJECTS,       {c_source},         {},         {},              {main_programs, ignore}, 	"{<object:t:r>.o, }" 	 },
  { CPP_OBJECTS,     {source},           {c_source}, {},              {main_programs, ignore}, 	"{<object:t:r>.o, }" 	 },
  { MAIN_S_OBJECTS,  {assembly},         {},         {main_programs}, {ignore},               	"{<object:t:r>.o, }" 	 },
  { MAIN_C_OBJECTS,  {c_source},         {},         {main_programs}, {ignore},               	"{<object:t:r>.o, }" 	 },
  { MAIN_CPP_OBJECTS,{source},           {c_source}, {main_programs}, {ignore},               	"{<object:t:r>.o, }" 	 },
  { C_EXECUTABLES,   {c_source},         {},         {main_programs}, {ignore},               	"{<object:t:r>, }"     	 },
  { CPP_EXECUTABLES, {source},           {c_source}, {main_programs}, {ignore},               	"{<object:t:r>, }" 	 },
  { DISASSEMBLED,    {source, assembly}, {},         {},              {ignore},	      		"{<object:t:r>.das, }"   },  
  
  { JAVA_SOURCES,         {java},        {},         {},              {ignore},                 "{<object:t>, }"              },
  { JAVA_CLASSES,         {java},        {},         {},              {ignore},                 "{<object:t:r>.class, }"      },
  { JAVA_GENERATE_CODE,   {java},        {},         {native},        {ignore},                 "{<object:t:r>.gen_code, }"   }
}
Makefile.Macros.Dynamic}

{Makefile.Macros.Dynamic_Block:
{
  { "# BEGIN DEPENDENCIES",
    "# END DEPENDENCIES",
    {source}, {}, {}, {ignore},
    "<set,do_include>"
  +   "<if><switch(makefile.dependency_mode)><eq>auto.info<then>"
  +     "false"
  +   "<elsif><switch(makefile.dependency_mode)><eq>man.info<then>"
  +     "false"
  +   "<else>"
  +     "true"
  +   "<endif>"
  + "<endset>"
  + "{<if><do_include><then>include $(DIR_MAKEFILES_PARENT)/<object:t:r>.dep<endif>,<nl>}" }
}
Makefile.Macros.Dynamic_Block}

# In the KRL string used in dynamic macro updates, the expanded format string must not include the
# separator string as a substring.

{Makefile.Macros.Switch:
{
  { COMPILER_OPTIONS,  "<if_switch(debugging,-g,)> "
        +              "<if><switch(cpp_compiler_kind)><eq>apex<then>"
        +                "<if_switch_eq(Command_Translation.Make,clearmake,-nofastpath,)> "
	+              "<endif>"
	+              "<if><switch_set(optimization_level)><then>"
	+                "<if><switch(cpp_compiler_kind)><eq>HP-3<then>"
	+                  " +O<switch(optimization_level)>"
	+                "<else>"
	+                  " -O<switch(optimization_level)>"
	+                "<endif>"
	+              "<endif> "
	+              "<if_switch_eq(profiling,prof,-p,)> "
	+              "<if_switch_eq(profiling,gprof,-pg,)> " },
  # Common options for the C and C++ compilers which are derived from
  # the switches DEBUGGING, OPTIMIZATION_LEVEL, PROFILING
  
  { LINKER_OPTIONS,  "<if_switch_eq(profiling,prof,-p,)> "
	+            "<if_switch_eq(profiling,gprof,-pg,)> " },
  # Common options for the C and C++ linkers which are derived from
  # the switch PROFILING
  
  { PTREPOSITORY_OPTION,  "<if><switch(cpp_compiler_kind)><eq>apex<then>"
	+                   " -ptr<switch(cpp_ptrepository_context)>"
	+                 "<elsif><switch(cpp_compiler_kind)><eq>SPARCompiler-5<then>"
	+                   "nil"
	+                 "<elsif><switch(cpp_compiler_kind)><eq>SPARCompiler-4<then>"
	+                   " -ptr<switch(cpp_ptrepository_context)>"
	+                 "<elsif><switch(cpp_compiler_kind)><eq>DEC-5<then>"
	+                   "-ptr <switch(cpp_ptrepository_context)>/<switch(cpp_ptrepository_name)>"
	+                 "<elsif><switch(cpp_compiler_kind)><eq>gcc<then>"
	+                   "nil"
	+                 "<else>"
	+                   "-ptr<switch(cpp_ptrepository_context)>/<switch(cpp_ptrepository_name)>"
	+                 "<endif>" },
  # The -ptr<pathname> option that should be used for a specific view

  { PTREPOSITORY_DIRECTORY,  "<if><switch(cpp_compiler_kind)><eq>apex<then>"
	+                      "<switch(cpp_ptrepository_context)>"
	+                    "<elsif><switch(cpp_compiler_kind)><eq>SPARCompiler-5<then>"
	+                      "nil"
	+                    "<elsif><switch(cpp_compiler_kind)><eq>SPARCompiler-4<then>"
	+                      "<switch(cpp_ptrepository_context)>"
	+                    "<else>"
	+                      "<switch(cpp_ptrepository_context)>/<switch(cpp_ptrepository_name)>"
	+                    "<endif>" },

  { MAKEFILE_DEPENDENCY_MODE, "<switch(Makefile.Dependency_Mode):l>" }

}
Makefile.Macros.Switch}

{Makefile.Macros.Import:
{
  { IMPORTED_C_INCLUDES,    "-ILinks/Imported_C   -ILinks/Imported -ILinks/Imported_Views" },
  { IMPORTED_CPP_INCLUDES,  "-ILinks/Imported_Cpp -ILinks/Imported -ILinks/Imported_Views" },
  
  { IMMEDIATE_C_INCLUDES,   "{<gen, order imports> resolve <view>/Links/Immediate_C/*,   -ILinks/Immediate_C/<object:t>, }   {<gen, order imports> resolve <view>/Links/Immediate/*, -ILinks/Immediate/<object:t>, }" },
  { IMMEDIATE_CPP_INCLUDES, "{<gen, order imports> resolve <view>/Links/Immediate_Cpp/*, -ILinks/Immediate_Cpp/<object:t>, } {<gen, order imports> resolve <view>/Links/Immediate/*, -ILinks/Immediate/<object:t>, }" },

  { JAVA_INCLUDES,     "<if><switch(java_home):b><and><switch(java_home):l><ne>nil<then>-I<switch(java_home)>/include <if><view'target_architecture><eq>sun4_solaris2<then>-I<switch(java_home)>/include/solaris<elsif><view'target_architecture><eq>alpha_osf1<then>-I<switch(java_home)>/include/java -I<switch(java_home)>/include/java/alpha<elsif><view'target_architecture><eq>hppa_hpux<then>-I<switch(java_home)>/include/hp-ux<elsif><view'target_architecture><eq>mips_irix5<then>-I<switch(java_home)>/include/irix<elsif><view'target_architecture><eq>mips_irix_n32<then>-I<switch(java_home)>/include/irix<elsif><view'target_architecture><eq>rs6k_aix<then>-I<switch(java_home)>/include/aix<elsif><view'target_architecture><eq>rs6k<then>-I<switch(java_home)>/include/aix<endif><endif>" }
}
Makefile.Macros.Import}  


###############################################################################
# Command Translation
#
# These properties define the meaning of build-oriented Apex commands for the
# different classes of objects.


# Options settings
# 
# The properties below define the way in which command line options are passed
# to the makefiles as the value of macros.  Most options are also represented
# by context switches which are automatically propagted into the makefiles 
# whenever the switch changes.  For those options the macro is only set on
# the command line if it was set as an option to the command.

{Command_Translation.Make:
	  "<if><view:e><eq>rss<and><switch(clearcase_enabled)><then>"
	+	"clearmake"
	+ "<else>"
	+	"make"
	+ "<endif>"
Command_Translation.Make}

Command_Translation.Makefile:     Makefile
# The root makefile

Command_Translation.Nodepfile:    .Makefiles.rtnl/Query.mk
# A alternative root makefile which does not include .dep dependency files

{Command_Translation.Make_Options:
	  " -r <if_switch(first_error,,-k)> <switch(make_options)>"
	+ "<if><view'host_architecture><eq>i386_linux2<then>"
	+     "<if><switch(Command_Translation.Make)><ne>clearmake<then>"
	+         " --no-print-directory -s "
	+     "<endif>"
	+ "<endif>"
Command_Translation.Make_Options}

{Command_Translation.Standard_Options: 
	  "EFFORT_ONLY=<switch(effort_only)> "	
	+ "FIRST_ERROR=<switch(first_error)> "
	+ "FORCE=<if_switch(force,force,)> "
	+ "VERBOSE=<switch(verbose)> "
Command_Translation.Standard_Options}

{Command_Translation.Compiler_Options:
          "<if><option_set(cpp_pre_options)><then>"
        +     "APEX_CPP_PRE_OPTIONS=<dq><switch(cpp_pre_options)><dq> "
        + "<endif>"
        + "<if><option_set(cpp_options)><then>"	
        +     "APEX_CPP_OPTIONS=<dq><switch(cpp_options)><dq> "	
        + "<endif>"
        + "<if><option_set(c_pre_options)><then>"
        +     "APEX_C_PRE_OPTIONS=<dq><switch(c_pre_options)><dq> "
        + "<endif>"
        + "<if><option_set(c_options)><then>"
        +     "APEX_C_OPTIONS=<dq><switch(c_options)><dq> "
        + "<endif>"
        + "<if><option_set(s_options)><then>"
        +     "APEX_S_OPTIONS=<dq><switch(s_options)><dq> "
        + "<endif>"
        + "<if><switch(Command_Translation.Make)><eq>clearmake<or>"
        +     "<option_set(debugging)><or>"
        +     "<option_set(optimization_level)><or>"
        +     "<option_set(profiling)><then> "
        +     "APEX_COMPILER_OPTIONS=<dq> "
        +                            "<if><switch(cpp_compiler_kind)><eq>apex<then>"
        +                              "<if_switch_eq(Command_Translation.Make,clearmake,-nofastpath,)> "
        +                            "<endif>"
        +                            "<if_switch(debugging,-g,)> "
        +                            "<if><option_set(optimization_level)><then>-O<switch(optimization_level)><endif> "
        +                            "<if_switch_eq(profiling,prof,-p,)> "
        +                            "<if_switch_eq(profiling,gprof,-pg,)> "
        +			     "<dq> "
        + "<endif>"
        + "<if><option_set(sienna)><then>"
	+     "APEX_SIENNA=<dq><switch(sienna)><dq> "
	+ "<endif>"
        + "<if><option_set(sienna_options)><then>"
	+     "APEX_SIENNA_OPTIONS=<dq><switch(sienna_options)><dq> "
	+ "<endif>"
        + "<if><option_set(dependencies)><then>"
	+     "APEX_DEPENDENCIES=<dq><switch(dependencies)><dq> "
	+ "<endif>"
        + "<if><option_set(dependencies_options)><then>"
	+     "APEX_DEPENDENCIES_OPTIONS=<dq><switch(dependencies_options)><dq> "
	+ "<endif>"
Command_Translation.Compiler_Options}

{Command_Translation.Closure_Options:
          "<if><option_set(link_closure_policy)><then>"	
        +     "APEX_LINK_CLOSURE_POLICY=<switch(link_closure_policy)> "	
        + "<endif>"
        + "<if><option_set(link_closure_languages)><then>"
        +     "APEX_LINK_CLOSURE_LANGUAGES=<switch(link_closure_languages)> "
        + "<endif>"
        + "COMPILE_CLOSURE=<view'compile_closure:b> "
        + "USE_EXISTING_CLOSURE=<switch(use_existing_closure)> "
        + "ALLOW_CLOSURE_ERROR=<switch(allow_closure_error)> "
        + "<if><option_set(configuration)><then>"
        +     "APEX_CONFIGURATION=<dq><switch(configuration)><dq> "
        + "<endif>"
        + "NO_MAIN_PROGRAM_COMPILATION=<switch(no_main_program_compilation):b> "
Command_Translation.Closure_Options}

{Command_Translation.Linker_Options:
	  "<if><option_set(cpp_link_pre_options)><then>"
	+     "APEX_CPP_LINK_PRE_OPTIONS=<dq><switch(cpp_link_pre_options)><dq> "
	+ "<endif>"
	+ "<if><option_set(cpp_link_options)><then>"
	+     "APEX_CPP_LINK_OPTIONS=<dq><switch(cpp_link_options)><dq> "
	+ "<endif>"
	+ "<if><option_set(cpp_link_mode)><then>"
	+     "APEX_CPP_LINK_MODE=<switch(cpp_link_mode)> "
	+ "<endif>"
	+ "<if><option_set(c_link_pre_options)><then>"	
	+     "APEX_C_LINK_PRE_OPTIONS=<dq><switch(c_link_pre_options)><dq> "
	+ "<endif>"
	+ "<if><option_set(c_link_options)><then>"
	+     "APEX_C_LINK_OPTIONS=<dq><switch(c_link_options)><dq> "
	+ "<endif>"
	+ "<if><option_set(c_link_mode)><then>"	
	+     "APEX_C_LINK_MODE=<switch(c_link_mode)> "
	+ "<endif>"
	+ "<if><option_set(purify)><then>"
 	+     "APEX_PURIFY=<switch(purify):b> "
	+ "<endif>"
	+ "<if><option_set(quantify)><then>"
 	+     "APEX_QUANTIFY=<switch(quantify):b> "
	+ "<endif>"
	+ "<if><option_set(purecoverage)><then>"
 	+     "APEX_PURECOVERAGE=<switch(purecoverage):b> "
	+ "<endif>"
	+ "<if><option_set(purelink)><then>"
 	+     "APEX_PURELINK=<switch(purelink):b> "
	+ "<endif>"
	+ "<if><option_set(enable_product)><then>"
 	+     "APEX_ENABLE_PRODUCT=<dq><switch(enable_product)><dq> "
	+ "<endif>"	
	+ "<if><option_set(check_link_dependencies)><then>"
 	+     "APEX_CHECK_LINK_DEPENDENCIES=<switch(check_link_dependencies):b> "
	+ "<endif>"	
Command_Translation.Linker_Options}

{Command_Translation.Dir_Options:
	  "<if><option_set(build_component)><then>"
	+     "APEX_BUILD_COMPONENT=<switch(build_component)> "
	+ "<endif>"
	+ "<if><option_set(build_phases)><then>"
	+     "APEX_BUILD_PHASES=<dq><switch(build_phases)><dq> "
	+ "<endif>"
	+ "<if><option_set(generation_target)><then>"
	+     "APEX_GENERATION_TARGET=<switch(generation_target)>"
	+ "<endif>"
Command_Translation.Dir_Options}

{Command_Translation.Java_Options:
	  "<if><option_set(javac_options)><then>"
	+     "APEX_JAVAC_OPTIONS=<dq><switch(javac_options)><dq> "
	+ "<endif>"
	+ "<if><option_set(java_classpath)><then>"
	+     "APEX_JAVA_CLASSPATH=<dq><switch(java_classpath)><dq> "
	+ "<endif>"
	+ "<if><option_set(javadoc_options)><then>"
	+     "APEX_JAVADOC_OPTIONS=<dq><switch(javadoc_options)><dq> "
	+ "<endif>"
Command_Translation.Java_Options}

{Command_Translation.All_Options: 
	  "<switch(Command_Translation.Standard_Options)> "
	+ "<switch(Command_Translation.Compiler_Options)> " 
	+ "<switch(Command_Translation.Linker_Options)> "
	+ "<switch(Command_Translation.Closure_Options)> "
	+ "<switch(Command_Translation.Dir_Options)> "
	+ "<switch(Command_Translation.Java_Options)> "
Command_Translation.All_Options}

#########################################
# Standard properties for invoking "make"
# 
# The properties below should always be used to invoke make
# The "Nodep" version can be used for operations which do not rely
# on dependency information.  The "Nodep" version is configured
# to always use standard make.
#
# MAKEFLAGS is unset so that command options which make sense to one
# make are not inherited by others. In particular, command line options
# for clearmake must not be inherited by others.

{Command_Translation.Standard_Make:
	  "unset MAKEFLAGS ; "
	+ "<switch(Command_Translation.Make)>"
        + " -f <switch(Command_Translation.Makefile)>"
        + " <switch(Command_Translation.Make_Options)>"
        + "<if><switch(Command_Translation.Make)><eq>clearmake<then>"
        +       " <if><switch(verbose)><then>-M -v<else>-M<endif>"
        + "<endif>"
        + " ROOT_MAKEFILE=<switch(Command_Translation.Makefile)>"
        + " <switch(Command_Translation.All_Options)>"
Command_Translation.Standard_Make}

{Command_Translation.Nodep_Make:
	  "unset MAKEFLAGS ; "
	+ "make"
        + " -f <switch(Command_Translation.Nodepfile)>"
        + " <switch(Command_Translation.Make_Options)>"
        + " ROOT_MAKEFILE=<switch(Command_Translation.Nodepfile)>"
        + " <switch(Command_Translation.All_Options)>"
Command_Translation.Nodep_Make}

###############################################################################
# Translation Table
#
# Most commands invoke make with a standard target, the name of the makefile is not specified
# and Makefile is assumed.
#
# The clean commands specify -f .Makefiles.rtnl/Query.mk in order to select that particular
# makefile.  This ensures that clean can run even if there is a problem with .dep files
# since those are not included into the Query.mk
#
# AIX requires that all options to make (eg, -r -k -f) preceed all of the macro settings.

{Command_Translation:
{
  { header,   check_rules,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.check_header, }" },  
  { header,   migrate,      "standard_migrate"},
  { header,   prepare,	    "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t>, }" },

  { source,   build,        "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.o, }" },
  { source,   clean,        "cd <dir> ; <switch(Command_Translation.Nodep_Make)>    {<object:t:r>.clean, }" },

  { source,   code,         "cd <dir> ; "
                          + "<switch(Command_Translation.Standard_Make)> {<object:t:r>.o, }" },

  { source,   check_rules,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.check_source, }" },
  { source,   dependencies, "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.depend, }" },
  { source,   disassemble,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> APEX_DISASSEMBLE_OPTIONS=<dq><switch(disassemble_options)><dq> DISASSEMBLE_OBJECTS=<dq>{<object:t:r>.o, }<dq> disassemble.objects.cpp_component" },
  { source,   link,         "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.o, } ; { if [ -f <object:t:r>.o ] ; then <switch(Command_Translation.Standard_Make)> <object:t:r> ; fi ;, }" },
  { source,   migrate,      "standard_migrate"},
  { source,   prepare,	    "cd <dir> ; <switch(Command_Translation.Standard_Make) {<object:t>, }" },

  { assembly, build,        "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.o, }" },
  { assembly, clean,        "cd <dir> ; <switch(Command_Translation.Nodep_Make)>    {<object:t:r>.clean, }" },
  { assembly, code,         "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.o, }" },

  { directory, analyze,     "cd <dir> ; <switch(Command_Translation.Standard_Make)> analyzed" },
  { directory, build,       "cd <dir> ; <switch(Command_Translation.Standard_Make)> built" },
  { directory, check_rules, "cd <dir> ; <switch(Command_Translation.Standard_Make)> checked_rules" },
  { directory, clean,       "cd <dir> ; <switch(Command_Translation.Nodep_Make)>    cleaned" },
  { directory, code,        "cd <dir> ; <switch(Command_Translation.Standard_Make)> coded" },
  { directory, deliver,     "cd <dir> ; <switch(Command_Translation.Standard_Make)> delivered" },
  { directory, dependencies,"cd <dir> ; <switch(Command_Translation.Standard_Make)> dependencies" },
  { directory, disassemble, "cd <dir> ; <switch(Command_Translation.Standard_Make)> disassembled" },
  { directory, finalize,    "cd <dir> ; <switch(Command_Translation.Standard_Make)> finalized" },
  { directory, generate,    "cd <dir> ; <switch(Command_Translation.Standard_Make)> generated" },
  { directory, link,	    "cd <dir> ; <switch(Command_Translation.Standard_Make)> coded ; <switch(Command_Translation.Standard_Make)> linked" },
  { directory, make,        "cd <dir> ; <switch(Command_Translation.Standard_Make)> <arguments>" },
  { directory, parse,       "cd <dir> ; <switch(Command_Translation.Standard_Make)> parsed" },
  { directory, prepare,     "cd <dir> ; <switch(Command_Translation.Standard_Make)> prepared" },
  { directory, pretty_print,"cd <dir> ; <switch(Command_Translation.Standard_Make)> pretty_printed" },
  { directory, test,        "cd <dir> ; <switch(Command_Translation.Standard_Make)> tested" },

  { view,      clean,       "cd <view> ; "
                          + "<switch(Command_Translation.Nodep_Make)> cleaned " },

  { view,      dependencies,"cd <dir> ; "
                          + "<switch(Command_Translation.Standard_Make)> dependencies ; "
                          + " rm -f ./.Makefiles.rtnl/cleaned_dependencies " },
                          
  { view,      code,        "cd <view> ; "
                          + "<switch(Command_Translation.Standard_Make)> coded" },

  { object_file,     disassemble,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> APEX_DISASSEMBLE_OPTIONS=<dq><switch(disassemble_options)><dq> DISASSEMBLE_OBJECTS=<dq>{<object:t>, }<dq> disassemble.objects.cpp_component" },

  { disasm,          disassemble,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> APEX_DISASSEMBLE_OPTIONS=<dq><switch(disassemble_options)><dq> DISASSEMBLE_OBJECTS=<dq>{<object:t:r>.o, }<dq> disassemble.objects.cpp_component" },

  { archive_library, code,  "cd <view> ; <switch(Command_Translation.Standard_Make)> coded" },
  { shared_library,  code,  "cd <view> ; <switch(Command_Translation.Standard_Make)> coded" },

  { closure,         build, "cd <dir> ; <switch(Command_Translation.Standard_Make)> COMPILE_CLOSURE=false USE_EXISTING_CLOSURE=true {<object:t:r>, }" },
  { closure,         code,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> COMPILE_CLOSURE=false USE_EXISTING_CLOSURE=true {<object:t:r>, }" },
  { closure,         link,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> COMPILE_CLOSURE=false USE_EXISTING_CLOSURE=true {<object:t:r>, }" },

  { java,            build,        "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.class, }" },
  { java,            clean,        "cd <dir> ; <switch(Command_Translation.Nodep_Make)>    {<object:t:r>.clean, }" },
  { java,            disassemble,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> APEX_DISASSEMBLE_OPTIONS=<dq><switch(disassemble_options)><dq> DISASSEMBLE_CLASSES=<dq>{<object:t:r>.java, }<dq> disassemble.classes.java_component" },
  { java,            code,         "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t:r>.class, }" },
  { java,            generate,     "cd <dir> ; <switch(Command_Translation.Standard_Make)> <if>(<switch(generation_target)><eq>code)<or>(<switch(generation_target)><eq>all)<then>{<object:t:r>.gen_code, }<endif>" },
  { java,   	     prepare,	   "cd <dir> ; <switch(Command_Translation.Standard_Make)> {<object:t>, }" },

  { java_class,      disassemble,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> APEX_DISASSEMBLE_OPTIONS=<dq><switch(disassemble_options)><dq> DISASSEMBLE_CLASSES=<dq>{<object:t:r>.java, }<dq> disassemble.classes.java_component" },
  { java_disasm,     disassemble,  "cd <dir> ; <switch(Command_Translation.Standard_Make)> APEX_DISASSEMBLE_OPTIONS=<dq><switch(disassemble_options)><dq> DISASSEMBLE_CLASSES=<dq>{<object:t:r>.java, }<dq> disassemble.classes.java_component" }

}
Command_Translation}


###############################################################################
# Utility Translation
#
# These properties define utility operations that are executed for various
# classes of objects. The supported utility operations are:
#	post_create	- invoked whenever an object is created
#	post_delete	- invoked whenever an object is deleted
#	post_maintain	- invoked whenever maintain -build is applied 
#	post_set_key	- invoked whenever a view containing an object is remodelled
#	
# Also the following utility operations are only applied to views
#	link_contribution_query
#	pre_freeze
# 
# Notes:
#   - post_maintain operations run Nodep_Make in order to avoid problems 
#     when .dep files are missing.

{Utility_Translation:
{
  { view, link_contribution_query,
	  "cd <view> ; "
	+ "<switch(Command_Translation.Standard_Make)> "
	+ "LINK_QUERY_RESULT=<dq><result><dq> "
	+ "LINK_QUERY_MODE=<if><link_mode><eq>default<then><view'switch(link_contribution_default_mode)><else><link_mode><endif> "
	+ "COMPILE_CLOSURE=<view'compile_closure:b> "
        + "NO_MAIN_PROGRAM_COMPILATION=true "
	+ "link_contribution.query"
  },

  { view, link_dominance_query,
	  "<set,shell,apex_shell> <view'switch(build_key)>/view_relationships.ash "
	+ "link_dominance -quiet -result <result> <link_closure_views>"
  },

  { view, post_create,
	  "<if><object'source><and><view'switch(cpp_compiler_kind):l><eq>apex<then>"
	+   "<if><switch(goal):l><ne>archived<and><switch(goal):l><ne>source<then>"
	+      "cd <view> ; "
        +      "<key>/post_create_view <view'switch(build_key_version)> "
        +                             "<view'switch(cpp_compiler_home)> "
        +                             "<view'switch(cpp_compiler_variant)> "
   	+                             "<view'switch(cpp_compiler_kind)> "
        +                             "<switch(goal):l> <object> <object'source> "
	+   "<else>"
	+      "standard_view_post_create"
	+   "<endif>"
	+ "<endif>"
  },

  { directory, post_maintain,
	  "cd <dir> ; "
        + "<switch(Command_Translation.Nodep_Make)> "
	+ "maintained"
  },

  { source, post_create,
	"standard_source_post_create"
  },

  { source, post_maintain,
	  "cd <dir> ; "
	+ "<switch(Command_Translation.Nodep_Make)> "
	+ "{<object:t:r>.maintain}"
  },

  { source, post_set_key,
	"standard_source_post_create"
  },

  { source, post_delete,
	  "cd <dir> ; "
	+ "<switch(Command_Translation.Nodep_Make)> "
	+ "CLEAN_ROOT={<object:t:r>} CLEAN_EXT={<object:t:e>} "
	+ "clean.deleted.cpp_component "
	+ "<;;> "
	+ "standard_source_post_delete"
  },
  
  { java, post_delete, 
          "cd <dir> ; "
	+ "<switch(Command_Translation.Nodep_Make)> "
	+ "CLEAN_ROOT={<object:t:r>} CLEAN_EXT={<object:t:e>} "
	+ "clean.deleted.java_component ; "
  }
  
}
Utility_Translation}


###############################################################################
# Message Handling
#
# These properties define special message handling characteristics associated
# with execution of the commands defined in the Command_Translation table
# above.

{Messages.Suppress:
{
# Many
  {"^\*\*\* Error.*code 1 *"},
  {"^.* is up to date."},
  
# Clearmake
  {"^$"},
  {"^clearmake: Error: Build script failed for .*"},
  {"^\".*\" not remade because of sub-target rebuild errors."},

# Sunos & Solaris
  {"^make: Warning: Target .* not remade because of errors"},
  {"^make: Fatal error: Command failed for target .*"},
  {"^Current working directory .*"},
  
  {"^ar: filename .* truncated to .*"},

# AIX
  {"^make: The error code from the last command is .*"},
  {"^ *The error code from the last failed command is .*"},
  {"^ *(continuing).*"},
  {"^Target .* did not make because of errors."},
  
# OSF 
  {"^.*Exit 1"},
  {"^.*(null command)"},
  
# HP (anchored to avoid performance problem on OSF/1)
  {"^.* not remade because of errors"}
}
Messages.Suppress}


###############################################################################
# Visibility Control
#
# These properties control which files are accessible through the Links
# directory.  They manage the file name spaces in both the Local and 
# Imported views.

{Visibility.Local:
{
  { {header, source, idl}, {}, {}, {main_programs, ignore} }
}
Visibility.Local}

{Visibility.Description.Default:
{
  {"* ."},
  {"* Links/Local"},

  {"* Links/Local <subsystem:t:r>"},
  {"* .           <subsystem:t:r> Imported_Views"}
}  
Visibility.Description.Default}


###############################################################################
# Cross Compiler Customization Properties

include_property_optional: <view'model>/Cross_Compilation.prop


###############################################################################
# Special Add-ins

 

 

###############################################################################
# User Customization Properties

include_property_optional: <view>/Policy/Customization.prop
