# +--------------------------------------------------------------------------+
# | Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
# | This software contains proprietary and confidential information of       |
# | Rational and its suppliers. Use, disclosure or reproduction is           |
# | prohibited without the prior express written consent of Rational.        |
# +--------------------------------------------------------------------------+
# 
################################################################################
# User Customization Makefile
#
# The makefile is provided for user customization in individual views.
#
# There are three types of customization which can be entered into this file:
#
#    1. Standard targets for views can be modified by setting then
#	PRIMARY_*_TARGET, CUSTOM_*_PRECONDITION and CUSTOM_*_ACTION macros.
#
#    2. The entire meaning of building a view can be changed by introducing
#       new build policy targets that are used in conjunction with the
#	BUILD_POLICY switch in the switch file.  The targets needed to 
#	define a "custom" build policy are provided below.
#
#    3. Additional rules and dependencies can be introduced which are
#       invoked directly by the user (perhaps through "apex make").
#	These rules may be used to cause the creation of C and C++ source
#	files when the "apex prepare" command is run.  Sample Yacc rules
#	are include below.


################################################################################
################################################################################
# Standard Target Customization
#
# Macros and Rules for adding additional preconditions or actions to the
# standard rules associated with views and directories, but not individual files
#
# The value of each _PRECONDITION macro is used to define an additional
# dependency for the associated standard targets.  In particular, the value of
# each _PRECONDITION macro can be used to define a target which will be made
# as part of making the associated standard target.
#
# The value of each _ACTION macro is used to define an additional action for
# the associated standard targets.  In particular, the value of an _ACTION
# macro will be executed after all other actions associated with the standard
# target have been executed.

CUSTOM_ANALYZE_PRECONDITION	 =
CUSTOM_ANALYZE_ACTION		 =

CUSTOM_BUILD_PRECONDITION	 =
CUSTOM_BUILD_ACTION		 =

CUSTOM_CHECK_RULES_PRECONDITION	 =
CUSTOM_CHECK_RULES_ACTION	 =

CUSTOM_CLEAN_PRECONDITION	 =
CUSTOM_CLEAN_ACTION		 =

CUSTOM_CODE_PRECONDITION 	 =
CUSTOM_CODE_ACTION	 	 =

CUSTOM_DELIVER_PRECONDITION	 =
CUSTOM_DELIVER_ACTION		 =

CUSTOM_DEPENDENCIES_PRECONDITION =
CUSTOM_DEPENDENCIES_ACTION	 =

CUSTOM_DISASSEMBLE_PRECONDITION  =
CUSTOM_DISASSEMBLE_ACTION	 =

CUSTOM_FINALIZE_PRECONDITION	 =
CUSTOM_FINALIZE_ACTION		 =

CUSTOM_GENERATE_PRECONDITION 	 =
CUSTOM_GENERATE_ACTION		 =

CUSTOM_LINK_PRECONDITION	 =
CUSTOM_LINK_ACTION		 =

CUSTOM_MAINTAIN_PRECONDITION	 =
CUSTOM_MAINTAIN_ACTION		 =

CUSTOM_PARSE_PRECONDITION	 =
CUSTOM_PARSE_ACTION		 =

CUSTOM_PREPARE_PRECONDITION 	 =
CUSTOM_PREPARE_ACTION		 =

CUSTOM_PRETTY_PRINT_PRECONDITION =
CUSTOM_PRETTY_PRINT_ACTION	 =

CUSTOM_TEST_PRECONDITION	 =
CUSTOM_TEST_ACTION		 =

################################################################################
# The following can be used to redefine the standard meaning of operations.
# To set the value, uncomment the macro and set a value which should be
# a makefile target.  Note that if the macro is uncommented and
# not set to anything then the corresponding command will not do anything.

# PRIMARY_ANALYZE_TARGET	= 
# PRIMARY_BUILD_TARGET		= 
# PRIMARY_CHECK_RULES_TARGET	=
# PRIMARY_CLEAN_TARGET		= 
# PRIMARY_CODE_TARGET		= 
# PRIMARY_DELIVER_TARGET	= 
# PRIMARY_DEPENDENCIES_TARGET	= 
# PRIMARY_DISASSEMBLE_TARGET	= 
# PRIMARY_FINALIZE_TARGET	=
# PRIMARY_GENERATE_TARGET	=
# PRIMARY_LINK_TARGET		= 
# PRIMARY_MAINTAIN_TARGET	= 
# PRIMARY_PARSE_TARGET		= 
# PRIMARY_PREPARE_TARGET	= 
# PRIMARY_PRETTY_PRINT_TARGET	= 
# PRIMARY_TEST_TARGET		= 

# PRIMARY_LINK_CONTRIBUTION_QUERY_TARGET 	= 
# PRIMARY_OBJECT_FILES_CONTENT_QUERY_TARGET 	= 


################################################################################
################################################################################
# Custom Build Policy
#
# Macros and rules for defining new build policies that control what is built
# in a view and how it is used in linking main programs.  The "custom_build"
# policy can be customized by defining actions for the targets below.
# Additional policies may also be specified by defining the required set of
# targets.

# The following targets are invoked when the associated command is
# run on the view.

custom_build.analyze:

custom_build.build:

custom_build.check_rules:

custom_build.clean:

custom_build.code:

custom_build.deliver:

custom_build.dependencies:

custom_build.disassemble:

custom_build.finalize:

custom_build.generate:

custom_build.link:

custom_build.maintain:

custom_build.parse:

custom_build.prepare:

custom_build.pretty_print:

custom_build.test:

# The following target is invoked when the view is making a contribution to
# the link of a main program.  If the view needs to contribute objects files,
# libraries, etc to the link, the names of those objects should be appended
# to the file named by the $(LINK_QUERY_RESULT) macro.
#
custom_build.link_contribution.query:

# The following target is invoked when the view is making supplying object
# files to be part of an archive that is being constructed in some other
# view.  The object files to be supplied should be appended to the file
# named by the $(CONTENT_QUERY_RESULT) macro.
#
custom_build.object_files_content.query:


###############################################################################
###############################################################################
# User Defined Rules
#
# Users may add any independent rules and dependencies here.  Targets
# introduced here may be invoked directly by running "make" or "apex make".
# In particular rules should be inserted below for the generation of
# C and C++ source code.  Sample rules are provided for the generation of
# C from Yacc.


###############################################################################
# Additional Object Files
#
# Object files that should be managed by the view, but which are not built
# by the normal rules can be named here.  These object files will be included
# into libraries and linkage queries like object files built by the normal
# rules.  These objects will be made whenever the view is coded.
# Rules to make these object files should be included in this
# Customization.mk file.  The macro below should not name object files
# which are built by the normal rules.

CUSTOM_OBJECTS	=


###############################################################################
# Sample Yacc rules
#
# Below are sample rules for running yacc to produce an object file
# from a yacc (.y) input file.  In order to generate C from Yacc
# the following steps are required:
#	1. Create the yacc file foo.y
#	2. run "apex prepare <view>"
#
# Note that some support for yacc is currently built into the build key
# properties.  This functionality will move into the Customization.sw

YACC				= yacc

YACC_OPTIONS			=

EXT				= c

.SUFFIXES: .$(EXT) .y .y_clean


.y.$(EXT):
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs preparation: $<	in $(APEX_DIRECTORY)";	      \
	     exit 0 ;							      \
	 fi ;								      \
	 $(UMASK) ;							      \
	 $(NOTE) "Preparing $*.$(EXT) from $< in $(APEX_DIRECTORY)" ;	      \
	 CMD="$(YACC) $(YACC_OPTIONS) $<";				      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 if $$CMD ; then						      \
	     $(MV) y.tab.c $*.$(EXT) ;					      \
	     $(POS) "Preparing $*.$(EXT) from $<" $(SUCCESS_IN_DIR) ;	      \
	 else								      \
	     $(NEG) "Preparing $*.$(EXT) from $<" $(FAILURE_IN_DIR) ;	      \
	     $(RM) y.tab.c ;						      \
	     exit 1 ;							      \
	 fi

.y.y_clean:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs cleaning: $< in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 $(NOTE) "Cleaning $< in $(APEX_DIRECTORY)" ;                         \
	 CMD="$(RM) $*.o $*.$(EXT) $*.das" ;				      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 if $$CMD ; then 						      \
	     $(POS) "Cleaning $<" $(SUCCESS_IN_DIR) ;			      \
	 else								      \
	     $(NEG) "Cleaning $<" $(FAILURE_IN_DIR) ;		              \
	     exit 1 ;							      \
	 fi
