# +--------------------------------------------------------------------------+
# | Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
# | This software contains proprietary and confidential information of       |
# | Rational and its suppliers. Use, disclosure or reproduction is           |
# | prohibited without the prior express written consent of Rational.        |
# +--------------------------------------------------------------------------+
# 
###############################################################################
# Definitions Makefile
#
# This makefile contains the common definitions that are available to all
# other makefiles.

VERSION			= $(APEX_KEY_VERSION)
#
# The version of the build key which determines the protocol for
# the interfaces in build key.  At various places the version in
# makefile is checked against the version in the scripts in the 
# build key.


###############################################################################
# Parameters
#
# These macros are used for passing parameters into the makefile.
# The value specifies the default if it is not overridden.

ALLOW_CLOSURE_ERROR	= false
#
# Specifies that a composite object (eg, an executable or library) should
# be built even if there was an error while compiling a component file.

EFFORT_ONLY		= false
#
# Specifies that the target is not actually to be made, rather the full
# name of the source file (or view) to be compiled will be emitted.
# Note that this only applies to compilation targets.

FIRST_ERROR		= false
#
# Specifies that the operation should stop when the first error is detected.
# Otherwise the operation should continue.  This usually implies that
# -k has also been set on the invocation of make.

FORCE			=
#
# When set to the value "force", will cause targets to be remade even
# when not out of date.

COMPILE_CLOSURE		= true
#
# Used during linking to determine if the closure of the program
# should be compiled.  When set to "true" the closure is compiled
# otherwise the closure is not compiled prior to linking.

USE_EXISTING_CLOSURE	= false
#
# Used during linking to cause the existing <main>.closure file to
# be used rather that recomputing it.  Also used when building libraries
# to cause the existing <lib>.files file to be used rather than
# recomputing it.  

NO_MAIN_PROGRAM_COMPILATION = false
#
# Used during coding to determine whether or not main program files will be
# compiled when an entire directory or view is being compiled.

VERBOSE			= false
#
# When set to true, additional messages are emitted.

LINK_QUERY_MODE		= dynamic_or_static
#
# When doing a link query the determines whether the static or the
# dynamic contribution is returned.
# Possible values are: static | dynamic | dynamic_or_static

LINK_QUERY_RESULT	=
#
# The rules associated with "link_contribution.query" will write the
# the result into this file.

CONTENT_QUERY_RESULT	=
#
# The rules associated with "object_files_content.query" will write the
# result into this file.

ROOT_MAKEFILE		= Makefile
#
# When recursive calls are made, this specifies the makefile that will
# be used.

STANDARD_OPTION_SETTINGS = EFFORT_ONLY=$(EFFORT_ONLY) FORCE=$(FORCE) VERBOSE=$(VERBOSE) FIRST_ERROR=$(FIRST_ERROR)
#
# The values of the standard macro settings above.

###############################################################################
# Standard Definitions

SHELL	= /bin/sh

CAT 	= cat
CMP	= cmp
CP	= cp
GREP	= fgrep
MAKE	= $(APEX_MAKE_PROG)
MAKE2	= make
# The second definition is for rules that should always use the standard
# make program even when used on ClearCase views.  Currently, this only applies
# to the clean operations.  In general, it should apply to all operations
# that use "Nodep_Make" in the command translations specified by Properties.sw.

MV	= mv
RM	= rm -rf
SED	= sed
TOUCH   = touch
UMASK	= umask $(APEX_UMASK)

ECHO 	= echo

NOTE	= echo "---"
POS	= echo "+++"
NEG	= echo "++*"
UNN	= echo "++-"
ERR	= echo "***"
WARN	= echo "!!!"
EFFORT	= echo "   "

TRACE   = $(NOTE) "Running:"


ALWAYS_OBSOLETE:

force:


###############################################################################
# Suffixes
#
# All supported suffixes are defined by the other makefiles

.SUFFIXES:


###############################################################################
# Clearmake special target rules (really switches)
#
# Do not check the following dependencies for reuse in order to prevent
# unnecessary recompilations.

.DEPENDENCY_IGNORED_FOR_REUSE: .Makefiles.rtnl/%

.DEPENDENCY_IGNORED_FOR_REUSE: %.reg

.DEPENDENCY_IGNORED_FOR_REUSE: Templates/%.state
.DEPENDENCY_IGNORED_FOR_REUSE: Templates/Template.opt
.DEPENDENCY_IGNORED_FOR_REUSE: Templates/Template.system

# Unfortunately, the following pattern matching rule is not supported
# by clearmake, so have to use the other more general rules instead.
# .DEPENDENCY_IGNORED_FOR_REUSE: .Rational/...
.DEPENDENCY_IGNORED_FOR_REUSE: heapmap
.DEPENDENCY_IGNORED_FOR_REUSE: %.cnh
.DEPENDENCY_IGNORED_FOR_REUSE: %.cno
.DEPENDENCY_IGNORED_FOR_REUSE: %.hmh
.DEPENDENCY_IGNORED_FOR_REUSE: %.hmo
.DEPENDENCY_IGNORED_FOR_REUSE: %.lin
.DEPENDENCY_IGNORED_FOR_REUSE: %.reg
.DEPENDENCY_IGNORED_FOR_REUSE: %.sw
.DEPENDENCY_IGNORED_FOR_REUSE: %.times

.DEPENDENCY_IGNORED_FOR_REUSE: Makefile
.DEPENDENCY_IGNORED_FOR_REUSE: %.closure


# Do not create sibling derived objects for the following files since
# they are of no real benefit until full wink-in can be supported and
# until then they would just add additional unnecessary overhead.
# Also, some of these files should never be derived objects anyway
# (such as .Makefiles.rtnl/Directory_Content) since they should never
# be winked into another view.
 
.NO_DO_FOR_SIBLING: .Makefiles.rtnl/%
 
.NO_DO_FOR_SIBLING: Policy/%.reg

.NO_DO_FOR_SIBLING: Templates/%.state
.NO_DO_FOR_SIBLING: Templates/Template.opt
.NO_DO_FOR_SIBLING: Templates/Template.system

# Unfortunately, the following pattern matching rule is not supported
# by clearmake, so have to use the other more general rules instead.
# .NO_DO_FOR_SIBLING: .Rational/...
.NO_DO_FOR_SIBLING: heapmap
.NO_DO_FOR_SIBLING: %.cnh
.NO_DO_FOR_SIBLING: %.cno
.NO_DO_FOR_SIBLING: %.hmh
.NO_DO_FOR_SIBLING: %.hmo
.NO_DO_FOR_SIBLING: %.lin
.NO_DO_FOR_SIBLING: %.reg
.NO_DO_FOR_SIBLING: %.sw
.NO_DO_FOR_SIBLING: %.times
 
.NO_DO_FOR_SIBLING: Makefile
.NO_DO_FOR_SIBLING: %.closure

 
###############################################################################
# Apex Command Macros

APEX		        = apex

FILTERS			= note+positive+negative+warning+error+internal_error+not_required

REQUIRED_OPTIONS	= -canonical_prefix +output_window +output_message

CMD_OPTIONS		= $(REQUIRED_OPTIONS) -output_filter $(FILTERS)

PREPARE			= $(APEX) prepare  $(CMD_OPTIONS)

COMPILE 		= $(APEX) compile  $(CMD_OPTIONS)

LINK 			= $(APEX) link     $(CMD_OPTIONS)

GENERATE		= $(APEX) generate $(CMD_OPTIONS)

COPY		        = $(APEX) copy     $(CMD_OPTIONS)

MAINTAIN		= $(APEX) maintain $(CMD_OPTIONS) -builder

APEX_MAKE 		= $(APEX) make     $(CMD_OPTIONS)

APEX_SHELL		= apex_shell


###############################################################################
# Standard Name Macros

ARCHIVED_LIB		= $(APEX_DIRECTORY)/$(APEX_LIBRARY_NAME)

ARCHIVED_LIB_FILES	= $(ARCHIVED_LIB).closure

SHARED_LIB		= $(APEX_DIRECTORY)/$(APEX_SHARED_LIBRARY_NAME)

SHARED_LIB_FILES	= $(SHARED_LIB).closure

NO_OBJECTS_FOR_LIB_FLAG = $(APEX_VIEW)/.Makefiles.rtnl/No_Objects_For_Library.flag


COMPILATION_CFG	    	= $(APEX_VIEW)/.Rational/Compilation/Compilation.cfg

EXPORT_SETS		= $(APEX_VIEW)/.Rational/Compilation/Compilation.export_sets

EXPLICITS_CFG		= $(APEX_VIEW)/.Rational/Imports_Control/Explicits.cfg

MUTUALS_CFG		= $(APEX_VIEW)/.Rational/Imports_Control/Mutual_Explicits.cfg

EXPLICITS_EXPORT_SETS	= $(APEX_VIEW)/.Rational/Imports_Control/Explicits.export_sets



###############################################################################
# Build Support Macros
#
# These define the support operations that are used.

BUILD_SUPPORT	   	= Apex_Build_Support -version $(VERSION)


###############################################################################
# Common success and failure message tails

SUCCESS_IN_DIR		= "in $(APEX_DIRECTORY) was successful"
FAILURE_IN_DIR		= "in $(APEX_DIRECTORY) was unsuccessful"

SUCCESS_FROM_DIR	= "from $(APEX_DIRECTORY) was successful"
FAILURE_FROM_DIR	= "from $(APEX_DIRECTORY) was unsuccessful"

SUCCESS_IN_VIEW		= "in $(APEX_VIEW) was successful"
FAILURE_IN_VIEW		= "in $(APEX_VIEW) was unsuccessful"

NO_EFFECT_IN_VIEW	= "has no effect in $(APEX_VIEW)"


###############################################################################
# Primary Targets
#
# These define the primary targets that are executed for the specified
# build operation.  These macros are used in the Master.mk and are defined
# here to allow them to be overridden in Customization.mk

PRIMARY_ANALYZE_TARGET		= $(APEX_BUILD_POLICY).analyze
PRIMARY_BUILD_TARGET		= $(APEX_BUILD_POLICY).build
PRIMARY_CHECK_RULES_TARGET	= $(APEX_BUILD_POLICY).check_rules
PRIMARY_CLEAN_TARGET		= $(APEX_BUILD_POLICY).clean
PRIMARY_CODE_TARGET		= $(APEX_BUILD_POLICY).code
PRIMARY_DELIVER_TARGET		= $(APEX_BUILD_POLICY).deliver
PRIMARY_DEPENDENCIES_TARGET	= $(APEX_BUILD_POLICY).dependencies
PRIMARY_DISASSEMBLE_TARGET	= $(APEX_BUILD_POLICY).disassemble
PRIMARY_FINALIZE_TARGET		= $(APEX_BUILD_POLICY).finalize
PRIMARY_GENERATE_TARGET		= $(APEX_BUILD_POLICY).generate
PRIMARY_LINK_TARGET		= $(APEX_BUILD_POLICY).link
PRIMARY_MAINTAIN_TARGET		= $(APEX_BUILD_POLICY).maintain
PRIMARY_PARSE_TARGET		= $(APEX_BUILD_POLICY).parse
PRIMARY_PREPARE_TARGET		= $(APEX_BUILD_POLICY).prepare
PRIMARY_PRETTY_PRINT_TARGET	= $(APEX_BUILD_POLICY).pretty_print
PRIMARY_TEST_TARGET		= $(APEX_BUILD_POLICY).test

PRIMARY_LINK_CONTRIBUTION_QUERY_TARGET 	  = $(APEX_BUILD_POLICY).link_contribution.query
PRIMARY_OBJECT_FILES_CONTENT_QUERY_TARGET = $(APEX_BUILD_POLICY).object_files_content.query


###############################################################################
# Preparation Targets
#
# These macros name targets that are built when the "prepare" command is
# run in a directory whose build component is "cpp".

ADD_IN_PREPARED_CPP		=
ADD_IN_PREPARED_CPP_1		=
ADD_IN_PREPARED_CPP_2		=
ADD_IN_PREPARED_CPP_3		=
ADD_IN_PREPARED_CPP_4		=
ADD_IN_PREPARED_CPP_5		=
ADD_IN_PREPARED_CPP_6		=
ADD_IN_PREPARED_CPP_7		=
ADD_IN_PREPARED_CPP_8		=
ADD_IN_PREPARED_CPP_9		=


###############################################################################
# Clean Objects
#
# These macros name objects that will be deleted when the "clean" command is
# run in a directory whose build component is "cpp".  The objects named
# by these macros will usually include the objects named in the corresponding
# ADD_IN_PREPARE_CPP macros.

ADD_IN_CLEAN_CPP		=
ADD_IN_CLEAN_CPP_1		=
ADD_IN_CLEAN_CPP_2		=
ADD_IN_CLEAN_CPP_3		=
ADD_IN_CLEAN_CPP_4		=
ADD_IN_CLEAN_CPP_5		=
ADD_IN_CLEAN_CPP_6		=
ADD_IN_CLEAN_CPP_7		=
ADD_IN_CLEAN_CPP_8		=
ADD_IN_CLEAN_CPP_9		=


###############################################################################
# Supported Policies and Components
#
# These macros list the values of BUILD_POLICY and BUILD_COMPONENT
# which the makefiles support.  The values of the macros can be overriden
# in Customization.mk to add or delete value.

SUPPORTED_BUILD_POLICIES 	= object_files library layer_library closure_library mutual_library mutual_member external_library ptrepository custom_build

SUPPORTED_BUILD_COMPONENTS	= cpp java java_cpp
