# +--------------------------------------------------------------------------+
# | Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
# | This software contains proprietary and confidential information of       |
# | Rational and its suppliers. Use, disclosure or reproduction is           |
# | prohibited without the prior express written consent of Rational.        |
# +--------------------------------------------------------------------------+
# 
###############################################################################
# Master Makefile
#
# This makefile contains the master rules that are defined for all directories.
#
# Notes
# -----
# In various places targets named DUMMYn are used to prevent syntax errors
# when macros are used to name targets and its possible that the macros
# are empty.
#
# Switches are checked (and updated) in Apex prior to invoking make.
# This implies that recursive compilation of subdirectories or other views
# should not call make directly but should go through an Apex compile command
# to ensure that switches are propagated.

###############################################################################
# Build Key Operations
#
# These are all the operations that are invoked from the build key.
# All the operations take 4 standard arguments


###############
# cpp component

STD_KEY_ARGS	   = $(VERSION) 					\
		     $(APEX_CPP_COMPILER_HOME)	 			\
		     $(APEX_CPP_COMPILER_VARIANT) 			\
		     $(APEX_CPP_COMPILER_KIND)

SIENNA_ARGS	   = $(VERSION) 					\
		     $(APEX_SIENNA_HOME)

md		   = $(APEX_KEY)/md $(STD_KEY_ARGS)
MD		   = $(APEX_KEY)/MD $(STD_KEY_ARGS)

sienna		   = $(APEX_KEY)/sienna $(SIENNA_ARGS)

cc		   = $(APEX_KEY)/cc $(STD_KEY_ARGS)
CC		   = $(APEX_KEY)/CC $(STD_KEY_ARGS)

ld		   = $(APEX_KEY)/ld $(STD_KEY_ARGS)
LD		   = $(APEX_KEY)/LD $(STD_KEY_ARGS)
Rld		   = $(APEX_KEY)/Rld $(STD_KEY_ARGS)
RLD		   = $(APEX_KEY)/RLD $(STD_KEY_ARGS)

as		   = $(APEX_KEY)/as $(STD_KEY_ARGS)

das		   = $(APEX_KEY)/das $(STD_KEY_ARGS)

create_library     = $(APEX_KEY)/create_library $(STD_KEY_ARGS)

check_rules	   = $(APEX_KEY)/check_rules $(STD_KEY_ARGS)

clean		   = $(APEX_KEY)/clean $(STD_KEY_ARGS)

compiler_mgr  	   = $(APEX_KEY)/compiler_mgr $(STD_KEY_ARGS)

tmpl_mgr  	   = $(APEX_KEY)/template_repository_mgr $(STD_KEY_ARGS)

maintain 	   = $(APEX_KEY)/maintain $(STD_KEY_ARGS)

check_build_config = $(APEX_KEY)/check_build_config.ash

get_temp	   = $(APEX_KEY)/get_temp_file

################
# java component

JAVA_KEY_ARGS   = $(VERSION) $(APEX_JAVA_HOME)

JAVAC           = $(APEX_KEY)/JAVAC $(JAVA_KEY_ARGS)

JAVAH		= $(APEX_KEY)/JAVAH $(JAVA_KEY_ARGS)

JAVAP		= $(APEX_KEY)/JAVAP $(JAVA_KEY_ARGS)

JAVADOC		= $(APEX_KEY)/JAVADOC $(JAVA_KEY_ARGS)


###############################################################################
# Option Settings for Compiling

###############
# cpp component

C_OPTIONS	= $(APEX_C_PRE_OPTIONS) \
		  $(C_INCLUDES) \
		  $(APEX_COMPILER_OPTIONS) \
		  $(APEX_C_OPTIONS) \
		  $(GLOBAL_C_OPTIONS)

CPP_OPTIONS	= $(APEX_CPP_PRE_OPTIONS) \
		  $(CPP_INCLUDES) \
		  $(APEX_COMPILER_OPTIONS) \
		  $(APEX_CPP_OPTIONS) \
		  $(GLOBAL_CPP_OPTIONS)

S_OPTIONS	= $(APEX_S_OPTIONS)

# Targets to query the options

c_options.query:
	@$(ECHO) $(C_OPTIONS) ; \
	 $(compiler_mgr) $(APEX_VIEW) query_c_options

cpp_options.query:
	@if [ "$(APEX_PTREPOSITORY_OPTION)" != "nil" ] ; then		     \
	    $(ECHO) $(CPP_OPTIONS) $(APEX_PTREPOSITORY_OPTION) ;	     \
	 else								     \
	    $(ECHO) $(CPP_OPTIONS) ;					     \
	 fi ;								     \
	 $(compiler_mgr) $(APEX_VIEW) query_cpp_options

s_options.query:
	@$(ECHO) $(S_OPTIONS)

cpp_ptrepository_options.query:
	@if [ "$(APEX_PTREPOSITORY_OPTION)" != "nil" ] ; then		     \
	    $(ECHO) $(APEX_PTREPOSITORY_OPTION) >> $(CONTENT_QUERY_RESULT) ; \
	 fi

cpp_include_options.query:
	 @rel_view_includes="$(CPP_INCLUDES)" ;				      \
	 for i in $$rel_view_includes ; do				      \
	     tmp_full_view=`$(ECHO) $$i | $(SED) -e 's/^-ILinks/Links/'` ;    \
	     full_view_includes="-I$(APEX_VIEW)/$$tmp_full_view $$full_view_includes" ; \
	 done ;								      \
	 echo $$full_view_includes >> $(CONTENT_QUERY_RESULT)

################
# java component

JAVAC_OPTIONS   = $(APEX_JAVAC_OPTIONS)

JAVA_CLASSPATH  = $(APEX_JAVA_CLASSPATH)


###############################################################################
# Suffixes

.SUFFIXES: .H .hh .hpp .hxx .h .C .cc .cpp .cxx .c .s .o
.SUFFIXES: .analysis .clean .depend .maintain .check_source .check_header .das
.SUFFIXES: .java .class .gen_code .gen_doc .jdas


###############################################################################
# Objects that are managed
#
# The objects named in the $(OBJECTS) are used for the following purposes:
#   1. To name the targets which are built when the code command is run
#      in a directory with the cpp build component.
#   2. To name the object files that are returned for the link contribution
#      query of when the build policy is "object_files".
#   3. To name the object files that are returned for the content query
#      which is used to calculate the objects included in libraries.

OBJECTS		= $(APEX_OBJECTS) $(CUSTOM_OBJECTS)


###############################################################################
###############################################################################
# PREPARE OPERATION

###############
# cpp_component

CUSTOM_PREPARED_CPP_TARGETS = $(APEX_PREPARED_C_SOURCES) $(APEX_PREPARED_CPP_SOURCES) $(APEX_PREPARED_HEADERS)

ADD_IN_PREPARED_CPP_TARGETS = $(ADD_IN_PREPARED_CPP) $(ADD_IN_PREPARED_CPP_1) $(ADD_IN_PREPARED_CPP_2) $(ADD_IN_PREPARED_CPP_3) $(ADD_IN_PREPARED_CPP_4) $(ADD_IN_PREPARED_CPP_5) $(ADD_IN_PREPARED_CPP_6) $(ADD_IN_PREPARED_CPP_7) $(ADD_IN_PREPARED_CPP_8) $(ADD_IN_PREPARED_CPP_9)

prepare.files.cpp_component: $(CUSTOM_PREPARED_CPP_TARGETS) $(ADD_IN_PREPARED_CPP_TARGETS)

prepare.files.announce.cpp_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Preparing C/C++ files in directory $(APEX_DIRECTORY)";    \
	 fi

prepare.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Preparing subdirectories of $(APEX_DIRECTORY)" ;	      \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE) $(STANDARD_OPTION_SETTINGS) prepare.contents.cpp_component \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

prepare.contents.cpp_component: prepare.files.cpp_component prepare.subdirs.cpp_component

################
# java_component

prepare.files.java_component: $(APEX_PREPARED_JAVA_SOURCES)

prepare.files.announce.java_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Preparing Java files in directory $(APEX_DIRECTORY)";     \
	 fi

prepare.subdirs.java_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Preparing subdirectories of $(APEX_DIRECTORY)" ;	      \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE) $(STANDARD_OPTION_SETTINGS) prepare.contents.java_component \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

prepare.contents.java_component: prepare.files.java_component prepare.subdirs.java_component

##################
# standard targets

object_files.prepare: prepare.standard

library.prepare: prepare.standard

layer_library.prepare: prepare.standard

closure_library.prepare: prepare.standard

mutual_library.prepare: prepare.standard

mutual_member.prepare: prepare.standard

external_library.prepare: prepare.noop

ptrepository.prepare: prepare.noop

####

prepare.standard: prepare.announce_view prepare.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

prepare.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Preparing view $(APEX_VIEW)" ;   	    		      \
	 fi

prepare.noop:
	@$(NOTE) "The prepare operation" $(NO_EFFECT_IN_VIEW)

################################
# cpp component standard targets

prepare.object_files.cpp_component: prepare.contents.cpp_component

prepare.library.cpp_component: prepare.contents.cpp_component

prepare.layer_library.cpp_component: prepare.contents.cpp_component

prepare.closure_library.cpp_component: prepare.contents.cpp_component

prepare.mutual_library.cpp_component: prepare.contents.cpp_component

prepare.mutual_member.cpp_component: prepare.contents.cpp_component

prepare.external_library.cpp_component: prepare.noop

#################################
# java component standard targets

prepare.object_files.java_component: prepare.contents.java_component

prepare.library.java_component: prepare.contents.java_component

prepare.layer_library.java_component: prepare.contents.java_component

prepare.closure_library.java_component: prepare.contents.java_component

prepare.mutual_library.java_component: prepare.contents.java_component

prepare.mutual_member.java_component: prepare.contents.java_component

prepare.external_library.java_component: prepare.noop.java_component

##############################################
# combined java/cpp component standard targets
#
# For convenience this invokes the "generate" target of the java_component to
# "prepare" the C source and header files.

prepare.object_files.java_cpp_component: generate.object_files.java_component

prepare.library.java_cpp_component: generate.library.java_component

prepare.layer_library.java_cpp_component: generate.layer_library.java_component

prepare.closure_library.java_cpp_component: generate.closure_library.java_component

prepare.mutual_library.java_cpp_component: generate.mutual_library.java_component

prepare.mutual_member.java_cpp_component: generate.mutual_member.java_component

prepare.external_library.java_cpp_component:

###############################################################################
###############################################################################
# PARSE OPERATION

parse.noop:
	@$(NOTE) "The parse operation" $(NO_EFFECT_IN_VIEW)

object_files.parse: parse.noop

library.parse: parse.noop

layer_library.parse: parse.noop

closure_library.parse: parse.noop

mutual_library.parse: parse.noop

mutual_member.parse: parse.noop

external_library.parse: parse.noop

ptrepository.parse: parse.noop

###############################################################################
###############################################################################
# PRETTY_PRINT OPERATION

pretty_print.noop:
	@$(NOTE) "The pretty_print operation" $(NO_EFFECT_IN_VIEW)

object_files.pretty_print: pretty_print.noop

library.pretty_print: pretty_print.noop

layer_library.pretty_print: pretty_print.noop

closure_library.pretty_print: pretty_print.noop

mutual_library.pretty_print: pretty_print.noop

mutual_member.pretty_print: pretty_print.noop

external_library.pretty_print: pretty_print.noop

ptrepository.pretty_print: pretty_print.noop

###############################################################################
###############################################################################
# DEPENDENCY OPERATION

###############
# cpp component

.C.depend .cc.depend .cpp.depend .cxx.depend:
	@$(NOTE) "Updating dependencies for $< in $(APEX_DIRECTORY)" ;	      \
	 if [ "$(APEX_PTREPOSITORY_OPTION)" != "nil" ] ; then		      \
	    OPS="$(CPP_OPTIONS) $(APEX_PTREPOSITORY_OPTION)" ;		      \
	 else								      \
	    OPS="$(CPP_OPTIONS)" ;					      \
	 fi ;								      \
	 CMD="$(MD) $(APEX_MAKEFILE_DEPENDENCY_MODE) $$OPS $(APEX_DEPENDENCIES_OPTIONS) $(APEX_DIRECTORY)/$<" ;\
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; $$CMD

.c.depend:
	@$(NOTE) "Updating dependencies for $< in $(APEX_DIRECTORY)" ;	      \
	 CMD="$(md) $(APEX_MAKEFILE_DEPENDENCY_MODE) $(C_OPTIONS) $(APEX_DEPENDENCIES_OPTIONS) $(APEX_DIRECTORY)/$<" ;\
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; $$CMD

####

dependencies.c_files.cpp_component:
	@if [ "$(APEX_C_SOURCES)" != "" ] ; then			      \
	   $(NOTE) "Updating C dependencies in $(APEX_DIRECTORY)";	      \
	   CMD="$(md) $(APEX_MAKEFILE_DEPENDENCY_MODE) $(C_OPTIONS) $(APEX_DEPENDENCIES_OPTIONS) $(APEX_C_SOURCES)" ;\
	   if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	   $(UMASK) ; 							      \
	   if $$CMD ; then						      \
	       $(POS) "Updating C dependencies $(SUCCESS_IN_DIR)" ;	      \
	   else 							      \
	       $(NEG) "Updating C dependencies $(FAILURE_IN_DIR)" ;	      \
	   fi ;	   							      \
	 fi

dependencies.cpp_files.cpp_component:
	@if [ "$(APEX_CPP_SOURCES)" != "" ] ; then			      \
	   $(NOTE) "Updating C++ dependencies in $(APEX_DIRECTORY)";	      \
	 if [ "$(APEX_PTREPOSITORY_OPTION)" != "nil" ] ; then		      \
	    OPS="$(CPP_OPTIONS) $(APEX_PTREPOSITORY_OPTION)" ;		      \
	 else								      \
	    OPS="$(CPP_OPTIONS)" ;					      \
	 fi ;								      \
	   CMD="$(MD) $(APEX_MAKEFILE_DEPENDENCY_MODE) $$OPS $(APEX_DEPENDENCIES_OPTIONS) $(APEX_CPP_SOURCES)";\
	   if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	   $(UMASK) ; 							      \
	   if $$CMD ; then						      \
	       $(POS) "Updating C++ dependencies $(SUCCESS_IN_DIR)" ;	      \
	   else 							      \
	       $(NEG) "Updating C++ dependencies $(FAILURE_IN_DIR)" ;	      \
	   fi ;	   							      \
	 fi

####

dependencies.files.cpp_component: dependencies.c_files.cpp_component dependencies.cpp_files.cpp_component
	@if [ "$(APEX_CPP_SOURCES)" = "" -a "$(APEX_C_SOURCES)" = "" ] ; then \
	   $(UNN) "No C/C++ source files for dependency update in directory $(APEX_DIRECTORY)" ; \
	 fi

dependencies.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Updating C++ dependencies in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE) $(STANDARD_OPTION_SETTINGS) dependencies.contents.cpp_component \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

dependencies.contents.cpp_component: dependencies.files.cpp_component dependencies.subdirs.cpp_component

##################
# standard targets

object_files.dependencies: dependencies.standard

library.dependencies: dependencies.standard

layer_library.dependencies: dependencies.standard

closure_library.dependencies: dependencies.standard

mutual_library.dependencies: dependencies.standard

mutual_member.dependencies:  dependencies.standard

external_library.dependencies: dependencies.noop

ptrepository.dependencies: dependencies.noop

####

dependencies.standard: dependencies.announce_view dependencies.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

dependencies.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Updating dependencies in view $(APEX_VIEW)" ;           \
	 fi

dependencies.noop:
	@$(NOTE) "The dependencies operation" $(NO_EFFECT_IN_VIEW)

################################
# cpp component standard targets

dependencies.object_files.cpp_component: dependencies.contents.cpp_component

dependencies.library.cpp_component: dependencies.contents.cpp_component

dependencies.layer_library.cpp_component: dependencies.contents.cpp_component

dependencies.closure_library.cpp_component: dependencies.contents.cpp_component

dependencies.mutual_library.cpp_component: dependencies.contents.cpp_component

dependencies.mutual_member.cpp_component: dependencies.contents.cpp_component

dependencies.external_library.cpp_component:

#################################
# java component standard targets

dependencies.noop.java_component:
	@$(NOTE) "The dependencies operation for Java" $(NO_EFFECT_IN_VIEW)

dependencies.object_files.java_component: dependencies.noop.java_component

dependencies.library.java_component: dependencies.noop.java_component

dependencies.layer_library.java_component: dependencies.noop.java_component

dependencies.closure_library.java_component: dependencies.noop.java_component

dependencies.mutual_library.java_component: dependencies.noop.java_component

dependencies.mutual_member.java_component: dependencies.noop.java_component

dependencies.external_library.java_component:

##############################################
# combined java/cpp component standard targets

dependencies.object_files.java_cpp_component: dependencies.object_files.cpp_component

dependencies.library.java_cpp_component: dependencies.library.cpp_component

dependencies.layer_library.java_cpp_component: dependencies.layer_library.cpp_component

dependencies.closure_library.java_cpp_component: dependencies.closure_library.cpp_component

dependencies.mutual_library.java_cpp_component: dependencies.mutual_library.cpp_component

dependencies.mutual_member.java_cpp_component: dependencies.mutual_member.cpp_component

dependencies.external_library.java_cpp_component:


###############################################################################
###############################################################################
# ANALYSIS OPERATION

analyze.noop:
	@$(NOTE) "The analyze operation" $(NO_EFFECT_IN_VIEW)

object_files.analyze: analyze.noop

library.analyze: analyze.noop

layer_library.analyze: analyze.noop

closure_library.analyze: analyze.noop

mutual_library.analyze: analyze.noop

mutual_member.analyze: analyze.noop

external_library.analyze: analyze.noop

ptrepository.analyze: analyze.noop

###############################################################################
###############################################################################
# CODE OPERATION

###############
# cpp component

.c.o:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs coding: $< in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ;							      \
	 fi ;								      \
	 $(NOTE) "Coding $< in $(APEX_DIRECTORY)" ;			      \
	 if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then		      \
	     $(RM) $*.o ;						      \
	 fi ;								      \
	 CMD="$(cc) $(APEX_MAKEFILE_DEPENDENCY_MODE) $(C_OPTIONS) $(APEX_DIRECTORY)/$<" ;		      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; 							      \
	 if $$CMD ; then						      \
	     if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then	      \
	         $(POS) "Coding $< $(SUCCESS_IN_DIR)" ;			      \
	     fi;							      \
	 else								      \
	     if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then	      \
	         $(NEG) "Coding $< $(FAILURE_IN_DIR)" ;			      \
	     fi;							      \
	     exit 1 ;							      \
	 fi ;								      \
	 if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then		      \
	     if [ "$(APEX_SIENNA)" = true ] ; then                            \
	         $(NOTE) "Updating dependendencies and sienna for $< in $(APEX_DIRECTORY)" ;\
	         CMD="$(sienna) $(C_OPTIONS) $(APEX_SIENNA_OPTIONS) $(APEX_DIRECTORY)/$<" ;\
	         if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;      \
	         $(UMASK) ; $$CMD ;					      \
	     elif [ "$(APEX_DEPENDENCIES)" = true ] ; then		      \
		 $(NOTE) "Updating dependencies for $< in $(APEX_DIRECTORY)"; \
	 	 CMD="$(md) $(APEX_MAKEFILE_DEPENDENCY_MODE) $(C_OPTIONS) $(APEX_DEPENDENCIES_OPTIONS) $(APEX_DIRECTORY)/$<" ; \
	 	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;      \
	 	 $(UMASK) ; $$CMD ;					      \
	     fi ;							      \
	 fi

.s.o:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs coding: $< in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ;							      \
	 fi ; 								      \
	 $(NOTE) "Coding $< in $(APEX_DIRECTORY)" ;			      \
	 $(RM) $*.o ;							      \
	 CMD="$(as) $(S_OPTIONS) $(APEX_DIRECTORY)/$<" ;		      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ;						 	      \
	 if $$CMD ; then						      \
	     if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then	      \
	         $(POS) "Coding $< $(SUCCESS_IN_DIR)" ;			      \
	     fi;							      \
	     exit 0 ;							      \
	 else								      \
	     if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then	      \
	         $(NEG) "Coding $< $(FAILURE_IN_DIR)" ;			      \
	     fi;							      \
	     exit 1 ;							      \
	 fi

.C.o .cc.o .cpp.o .cxx.o:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs coding: $< in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 $(NOTE) "Coding $< in $(APEX_DIRECTORY)" ;			      \
	 if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then		      \
	     $(RM) $*.o ;						      \
	 fi ;								      \
	 if [ "$(APEX_PTREPOSITORY_OPTION)" != "nil" ] ; then		      \
	    OPS="$(CPP_OPTIONS) $(APEX_PTREPOSITORY_OPTION)" ;		      \
	 else								      \
	    OPS="$(CPP_OPTIONS)" ;					      \
	 fi ;								      \
	 CMD="$(CC) $(APEX_MAKEFILE_DEPENDENCY_MODE) $(APEX_CPP_COMPILE_TEMPLATE_INSTANTIATIONS) $$OPS $(APEX_DIRECTORY)/$<" ;		      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; 							      \
	 if $$CMD ; then						      \
	     if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then	      \
	         $(POS) "Coding $< $(SUCCESS_IN_DIR)" ;			      \
	     fi;							      \
	 else								      \
	     if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then	      \
	         $(NEG) "Coding $< $(FAILURE_IN_DIR)" ;			      \
	     fi;							      \
	     exit 1 ;							      \
	 fi ;								      \
	 if [ "$(APEX_CPP_COMPILER_KIND)" != "apex" ] ; then		      \
	     if [ "$(APEX_SIENNA)" = true ] ; then                            \
	         $(NOTE) "Updating dependencies and sienna for $< in $(APEX_DIRECTORY)" ;    \
	         CMD="$(sienna) $(CPP_OPTIONS) $(APEX_SIENNA_OPTIONS) $(APEX_DIRECTORY)/$<" ;\
	         if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;      \
	         $(UMASK) ; $$CMD ;					      \
	     elif [ "$(APEX_DEPENDENCIES)" = true ] ; then		      \
		 $(NOTE) "Updating dependencies for $< in $(APEX_DIRECTORY)"; \
	 	 CMD="$(MD) $(APEX_MAKEFILE_DEPENDENCY_MODE) $(CPP_OPTIONS) $(APEX_DEPENDENCIES_OPTIONS) $(APEX_DIRECTORY)/$<" ;	\
	 	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;      \
	 	 $(UMASK) ; $$CMD ;					      \
	     fi ;							      \
	 fi

####

code.if_true.no_main_cpp_component: $(OBJECTS)

code.if_false.no_main_cpp_component: $(OBJECTS) $(APEX_MAIN_OBJECTS)

code.files.cpp_component: code.files.announce.cpp_component code.if_$(NO_MAIN_PROGRAM_COMPILATION).no_main_cpp_component

code.files.announce.cpp_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Coding C/C++ files in $(APEX_DIRECTORY)" ; 		      \
	 fi

code.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Coding C/C++ files in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE) $(STANDARD_OPTION_SETTINGS) $(COMPILER_OPTION_SETTINGS) NO_MAIN_PROGRAM_COMPILATION=$(NO_MAIN_PROGRAM_COMPILATION) code.contents.cpp_component  \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

code.contents.cpp_component: code.files.cpp_component code.subdirs.cpp_component

# code.view.cpp_component will code the "normal" contents of the view
# and in addition update any template repository objects

code.view.cpp_component: code.contents.cpp_component
	@if [ "$(VERBOSE)" = true -a "$(APEX_CPP_COMPILE_TEMPLATE_INSTANTIATIONS)" = true ] ; then				      \
	    $(NOTE) "Updating template instantiations in view $(APEX_VIEW)" ; \
	 fi ;								      \
	 if [ "$(EFFORT_ONLY)" = true ] ; then exit ; fi ;		      \
	 regular_objs_content=`$(get_temp) reg_objs_content_$$$$` ;	      \
	 $(UMASK) ;							      \
	 $(TOUCH) $$regular_objs_content ;				      \
	 objects="$(OBJECTS)" ;						      \
	 for i in $$objects ; do					      \
	     if [ -f $$i ] ; then					      \
	         $(ECHO) $(APEX_DIRECTORY)/$$i >> $$regular_objs_content ;    \
	     fi ;							      \
	 done ;								      \
	 subdirs="$(APEX_SUBDIRS)" ;					      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	       (cd $$i ; 						      \
	        $(MAKE) CONTENT_QUERY_RESULT=$$regular_objs_content           \
	                subdir_object_files_content.query ; 		      \
	       ) ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ "$(APEX_CPP_COMPILE_TEMPLATE_INSTANTIATIONS)" = true ] ; then   \
	     if [ "$(APEX_PTREPOSITORY_DIRECTORY)" = "" ] ; then	      \
		ptr_dir="nil" ;						      \
	     else							      \
		ptr_dir="$(APEX_PTREPOSITORY_DIRECTORY)" ;		      \
	     fi ;							      \
	     CMD="$(tmpl_mgr) $(APEX_VIEW) update $$ptr_dir $$regular_objs_content nil $(CPP_OPTIONS)" ; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ; 	      \
	     $(UMASK) ;							      \
	     if $$CMD ; then						      \
		 : ; 							      \
	     else							      \
		 $(RM) $$regular_objs_content ;				      \
		 exit 1 ;						      \
	     fi ;							      \
	 fi ;								      \
	 $(RM) $$regular_objs_content

####

code.library_import_closure.cpp_component:
	@if [ "$(COMPILE_CLOSURE)" = false ] ; then exit 0; fi ;              \
	 if [ "$(USE_EXISTING_CLOSURE)" = true ] ; then exit 0; fi ;          \
	 if [ "$(EFFORT_ONLY)" = true ] ; then effortopt="-effort_only" ; fi ;\
	 if [ "$(FORCE)" != "" ] ;        then forceopt="-force" ; fi ;	      \
	 if [ "$(VERBOSE)" = true ] ;     then verboseopt="-verbose" ; fi ;   \
	 if [ "$(FIRST_ERROR)" = true ] ; then ferroropt="-first_error" ; fi ;\
	 if [ "$(NO_MAIN_PROGRAM_COMPILATION)" = true ] ; then nompcopt="-no_main_program_compilation" ; fi ; \
	 componentopt="-build_component $(APEX_BUILD_COMPONENT)" ;            \
	 noimportsopt="+imports" ;					      \
	 options="$$forceopt $$verboseopt $$effortopt $$ferroropt $$nompcopt $$componentopt $$noimportsopt" ; \
	 if [ "$(EFFORT_ONLY)" = true ] ; then				      \
	     $(NOTE) "Computing effort to compile library import closure" ;   \
	 else								      \
	     $(NOTE) "Compiling library import closure" ;		      \
	 fi ;								      \
	 if [ "$(APEX_BUILD_POLICY)" = layer_library ] ; then		      \
	     supcommand=get_explicit_imports ;				      \
	 elif [ "$(APEX_BUILD_POLICY)" = closure_library ] ; then	      \
	     supcommand=get_all_imports ;				      \
	 elif [ "$(APEX_BUILD_POLICY)" = mutual_library ] ; then	      \
	     supcommand=get_mutual_imports ;				      \
	 else								      \
	     exit 0 ;							      \
	 fi ;								      \
	 for i in `$(BUILD_SUPPORT) $$supcommand -commit` ; do 		      \
	     if [ -d $$i ] ; then 					      \
	       if [ ! -w $$i ] ; then 					      \
	           if [ "$(VERBOSE)" = true ] ; then			      \
	               $(NOTE) "Not coding read-only view $$i" ;	      \
	           fi ;							      \
	       elif $(COMPILE) $$options $$i ; then			      \
	           continue ;						      \
	       elif [ "$(FIRST_ERROR)" = true ] ; then			      \
	           exit 1 ;						      \
	       fi ;							      \
	     fi ;					 	              \
	 done

####

code.contents.if_closure.cpp_component: code.contents.if_$(COMPILE_CLOSURE).cpp_component

code.contents.if_false.cpp_component:

code.contents.if_true.cpp_component: code.contents.cpp_component

####

code.create_library.cpp_component:
	@if [ "$(APEX_DIRECTORY_CLASS)" = "Directory" ] ; then exit 0 ; fi ;  \
	 if [ "$(EFFORT_ONLY)" = true ] ; then exit 0 ; fi ; 		      \
	 $(NOTE) "Building library in $(APEX_VIEW)" ;			      \
	 if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" != "true" -a		      \
	      "$(APEX_CREATE_SHARED_LIBRARY)" != "true" ] ; then	      \
	     $(WARN) "Neither archived nor shared libraries have been requested" ; \
	     $(WARN) "No libraries will be built in $(APEX_VIEW)" ;	      \
	     exit 0 ;							      \
	 fi ;								      \
	 if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" = "true" -a		      \
	      "$(APEX_LIBRARY_NAME)" = "" ] ; then			      \
	     $(ERR)  "Unable to create archived library in $(APEX_VIEW) because the LIBRARY_NAME switch is not set" ; \
	     exit 1 ;							      \
	 fi ;								      \
	 if [ "$(APEX_CREATE_SHARED_LIBRARY)" = "true" -a		      \
	      "$(APEX_SHARED_LIBRARY_NAME)" = "" ] ; then		      \
	     $(ERR)  "Unable to create shared library in $(APEX_VIEW) because the SHARED_LIBRARY_NAME switch is not set" ; \
	     exit 1 ;							      \
	 fi ;								      \
         $(UMASK) ; 							      \
         archived_lib_new_files=`$(get_temp) archived_new_files_$$$$` ;	      \
	 shared_lib_new_files=`$(get_temp) shared_new_files_$$$$` ;	      \
	 use_existing_closure="false" ;					      \
	 if [ "$(USE_EXISTING_CLOSURE)" = "true" ] ; then		      \
	     use_existing_closure="true" ;				      \
	     if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" = "true" -a		      \
	     	  ! -f "$(ARCHIVED_LIB_FILES)" ] ; then			      \
	         use_existing_closure="false" ;				      \
	     fi ;							      \
	     if [ "$(APEX_CREATE_SHARED_LIBRARY)" = "true" -a		      \
	     	  ! -f "$(SHARED_LIB_FILES)" ] ; then			      \
	         use_existing_closure="false" ;				      \
	     fi ;						 	      \
	 fi ;								      \
	 if [ "$$use_existing_closure" = "true" ] ; then		      \
	     $(NOTE) "Library will be reconstructed from previous closure" ;  \
	     if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" = "true" ] ; then	      \
	         $(CP) $(ARCHIVED_LIB_FILES) $$archived_lib_new_files ;	      \
	     fi ;							      \
	     if [ "$(APEX_CREATE_SHARED_LIBRARY)" = "true" ] ; then	      \
	         $(CP) $(SHARED_LIB_FILES) $$shared_lib_new_files ;	      \
	     fi ;							      \
	 else								      \
	     $(NOTE) "Computing library closure" ;			      \
	     tmp_new_files=`$(get_temp) tmp_files_$$$$` ;		      \
	     lib_new_views=`$(get_temp) new_views_$$$$` ;		      \
	     shared_lib_new_files=$$archived_lib_new_files ;		      \
	     $(TOUCH) $$tmp_new_files $$lib_new_views ;			      \
	     case "$(APEX_BUILD_POLICY)" in 				      \
	         library) 						      \
	             $(ECHO) $(APEX_VIEW) >> $$lib_new_views ;		      \
	             ;;							      \
	         layer_library)						      \
	    	     $(BUILD_SUPPORT) get_explicit_imports -result $$lib_new_views -self ; \
	    	     ;;							      \
	    	 closure_library)					      \
	    	     $(BUILD_SUPPORT) get_all_imports -result $$lib_new_views -self ; \
	    	     ;;							      \
	    	 mutual_library)					      \
	   	     $(BUILD_SUPPORT) get_mutual_imports -result $$lib_new_views -self ; \
	   	     ;;							      \
	   	 *)							      \
	   	     ;;							      \
	     esac ;							      \
	     for i in `$(CAT) $$lib_new_views` ; do			      \
	         if [ -d $$i ] ; then					      \
	             cd $$i ;						      \
	             CMD="$(APEX_MAKE)					      \
				CONTENT_QUERY_RESULT=$$tmp_new_files	      \
				APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS=$(APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS) \
				VERBOSE=$(VERBOSE)			      \
				object_files_content.query";		      \
		      if $$CMD ; then					      \
			  : ;						      \
		      else						      \
			  $(RM) $$tmp_new_files $$lib_new_views ;	      \
			  $(NEG) "Building library" $(FAILURE_IN_DIR) ;       \
			  exit 1 ; 					      \
		      fi ;						      \
	         else							      \
	             $(ERR) "$(APEX_VIEW): Import $$i does not exist" ;	      \
	         fi ;							      \
	     done ; 				 		              \
	     $(SED) -e '/^ *$$/d' $$tmp_new_files > $$archived_lib_new_files ; \
	     $(RM) $$tmp_new_files $$lib_new_views ;			      \
	     $(NOTE) "library closure computed" ;			      \
	 fi ;								      \
	 if [ "$(FORCE)" = "force" ] ; then forceopt="-force" ; fi ;	      \
	 if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" = "true" ] ; then	      \
	     if [ -n "$(APEX_ARCHIVED_LIBRARY_OPTIONS)" ] ; then	      \
		archived_lib_options=`$(get_temp) archived_lib_options_$$$$` ; \
		$(ECHO) "$(APEX_ARCHIVED_LIBRARY_OPTIONS)" > $$archived_lib_options ; \
		arlib_opts="-archived_lib_options $$archived_lib_options" ;   \
	     fi ;							      \
	     archivedopts="-archived -archived_lib $(ARCHIVED_LIB) -archived_lib_files $(ARCHIVED_LIB_FILES) -archived_lib_files_new $$archived_lib_new_files $$arlib_opts" ; \
	 fi ;								      \
	 if [ "$(APEX_CREATE_SHARED_LIBRARY)" = "true" ] ; then	              \
	     if [ -n "$(APEX_SHARED_LIBRARY_OPTIONS)" ] ; then		      \
		shared_lib_options=`$(get_temp) shared_lib_options_$$$$` ;    \
		$(ECHO) "$(APEX_SHARED_LIBRARY_OPTIONS)" > $$shared_lib_options ; \
		shlib_opts="-shared_lib_options $$shared_lib_options" ;	      \
	     fi ;							      \
	     sharedopts="-shared -shared_lib $(SHARED_LIB) -shared_lib_files $(SHARED_LIB_FILES) -shared_lib_files_new $$shared_lib_new_files $$shlib_opts" ; \
	 fi ;								      \
	 viewopt="-view $(APEX_VIEW)" ;					      \
	 CMD="$(create_library) $$forceopt $$viewopt $$archivedopts $$sharedopts" ; \
	 if [ "$(VERBOSE)" = "true" ] ; then  $(TRACE) $$CMD ; fi ;	      \
	 if $$CMD ; then 						      \
	     $(RM) $$archived_lib_new_files $$shared_lib_new_files $$shared_lib_options ; \
	 else								      \
	     $(RM) $$archived_lib_new_files $$shared_lib_new_files $$shared_lib_options ; \
	     exit 1 ;							      \
	 fi

################
# java component

.java.class:
	@if [ "$(EFFORT_ONLY)" = true ] ; then                                \
	     $(EFFORT) "Needs coding: $< in $(APEX_DIRECTORY)" ;              \
	     exit 0 ;                                                         \
         fi ;                                                                 \
	 $(NOTE) "Coding $< in $(APEX_DIRECTORY)" ;                           \
	 CMD="$(JAVAC) $(JAVA_CLASSPATH) $(JAVAC_OPTIONS) $(APEX_DIRECTORY)/$<" ; \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
         $(UMASK) ;                                                           \
	 if $$CMD ; then                                                      \
	     $(POS) "Coding $<" $(SUCCESS_IN_DIR) ;                           \
         else                                                                 \
	     $(NEG) "Coding $<" $(FAILURE_IN_DIR) ;                           \
	     exit 1 ;                                                         \
	 fi


code.files.java_component: code.files.announce.java_component $(APEX_JAVA_CLASSES)

code.files.announce.java_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Coding Java files in $(APEX_DIRECTORY)" ; 		      \
	 fi

code.subdirs.java_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Coding Java files in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE) $(STANDARD_OPTION_SETTINGS) code.contents.java_component \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

code.contents.java_component: code.files.java_component code.subdirs.java_component

##################
# standard targets

object_files.code: code.standard

library.code: code.standard

layer_library.code: code.standard

closure_library.code: code.standard

mutual_library.code: code.standard

mutual_member.code: code.standard

external_library.code: code.noop

ptrepository.code: code.noop

####

code.standard: code.announce_view code.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

code.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Coding view $(APEX_VIEW)" ;   	   		      \
	 fi

code.noop:
	@$(NOTE) "The code operation" $(NO_EFFECT_IN_VIEW)

################################
# cpp component standard targets

code.object_files.cpp_component: code.view.cpp_component

code.library.cpp_component: code.contents.if_closure.cpp_component code.create_library.cpp_component

code.layer_library.cpp_component: code.contents.if_closure.cpp_component code.library_import_closure.cpp_component code.create_library.cpp_component

code.closure_library.cpp_component: code.contents.if_closure.cpp_component code.library_import_closure.cpp_component code.create_library.cpp_component

code.mutual_library.cpp_component: code.contents.if_closure.cpp_component code.library_import_closure.cpp_component code.create_library.cpp_component

code.mutual_member.cpp_component: code.contents.cpp_component

code.external_library.cpp_component:

#################################
# java component standard targets

code.object_files.java_component: code.contents.java_component

code.library.java_component: code.contents.java_component

code.layer_library.java_component: code.contents.java_component

code.closure_library.java_component: code.contents.java_component

code.mutual_library.java_component: code.contents.java_component

code.mutual_member.java_component: code.contents.java_component

code.external_library.java_component:

##############################################
# combined java/cpp component standard targets

code.object_files.java_cpp_component: code.object_files.java_component code.object_files.cpp_component

code.library.java_cpp_component: code.library.java_component code.library.cpp_component

code.layer_library.java_cpp_component: code.layer_library.java_component code.layer_library.cpp_component

code.closure_library.java_cpp_component: code.closure_library.java_component code.closure_library.cpp_component

code.mutual_library.java_cpp_component: code.mutual_library.java_component code.mutual_library.cpp_component

code.mutual_member.java_cpp_component: code.mutual_member.java_component code.mutual_member.cpp_component

code.external_library.java_cpp_component:

###############################################################################
###############################################################################
# LINK OPERATION

C_LINK_MODE		= $(APEX_C_LINK_MODE)

C_LINK_PRE_OPTIONS	= $(APEX_LINKER_OPTIONS) $(GLOBAL_C_LINK_PRE_OPTIONS) $(APEX_C_LINK_PRE_OPTIONS)

C_LINK_OPTIONS		= $(APEX_C_LINK_OPTIONS) $(GLOBAL_C_LINK_OPTIONS)

CPP_LINK_MODE		= $(APEX_CPP_LINK_MODE)

CPP_LINK_PRE_OPTIONS	= $(APEX_LINKER_OPTIONS) $(GLOBAL_CPP_LINK_PRE_OPTIONS) $(APEX_CPP_LINK_PRE_OPTIONS)

CPP_LINK_OPTIONS	= $(APEX_CPP_LINK_OPTIONS) $(GLOBAL_CPP_LINK_OPTIONS)

$(APEX_C_EXECUTABLES) DUMMY1: ALWAYS_OBSOLETE
	@if [ "$(EFFORT_ONLY)" = true ] ; then				      \
	     $(NOTE) "Effort required to link $@ in $(APEX_DIRECTORY)" ;      \
	 else								      \
	     $(NOTE) "Linking $@ in $(APEX_DIRECTORY)" ;		      \
	 fi ;								      \
	 if [ "$(EFFORT_ONLY)" != true -a ! -f $@.o ] ; then 		      \
	     $(NEG) "Unable to find main program object file $@.o" ; exit 1 ; \
	 fi ;								      \
	 if [ "$(APEX_CONFIGURATION)" != "" -a				      \
	      "$(APEX_LINK_CLOSURE_POLICY)" != "imports" ] ; then	      \
	     $(NOTE) "Linking with configuration $(APEX_CONFIGURATION)" ;     \
	 fi ;								      \
	 closurefile=$(APEX_DIRECTORY)/$@.closure ;			      \
	 tmpclosurefile=`$(get_temp) $@.closure.$$$$` ;			      \
	 altclosurefile=$$tmpclosurefile ;				      \
	 if [ "$(VERBOSE)" = true ] ; then verboseopt="-verbose" ; fi ;       \
	 if [ "$(FORCE)" != "" ] ; then forceopt="-force" ; fi ;              \
	 if [ "$(FIRST_ERROR)" = true ] ; then ferroropt="-first_error" ; fi ;\
	 if [ "$(EFFORT_ONLY)" = true ] ; then				      \
	     if [ "$(COMPILE_CLOSURE)" = true ] ; then			      \
	         $(NOTE) "Computing effort to compile closure" ;	      \
	         if $(BUILD_SUPPORT) get_linkage_closure 		      \
	                        -compile -effort_only			      \
	                        $$forceopt $$verboseopt $$ferroropt	      \
	                        -result $$tmpclosurefile	              \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(C_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(C_LINK_OPTIONS)"	      \
	                        -link_mode "$(C_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	         else							      \
	             $(RM) $$tmpclosurefile ;				      \
	             $(NEG) "Unable to compute closure effort" ;	      \
	             exit 1 ;						      \
	         fi ;							      \
	     fi ;							      \
	 elif [ "$(USE_EXISTING_CLOSURE)" = true ] ; then		      \
	     $(NOTE) "Using existing closure" ;			     	      \
	     if [ -f $$closurefile ] ; then				      \
	         altclosurefile=$$closurefile ;				      \
	     fi ;							      \
	     if $(BUILD_SUPPORT) get_linkage_closure 			      \
	                        $$verboseopt $$ferroropt -use_existing_closure\
	                        -result $$altclosurefile		      \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(C_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(C_LINK_OPTIONS)"	      \
	                        -link_mode "$(C_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	     else							      \
	         $(RM) $$tmpclosurefile ;				      \
	         $(NEG) "Unable to compute closure" ;			      \
	         exit 1 ;						      \
	     fi ;							      \
	 elif [ "$(COMPILE_CLOSURE)" = false ] ; then			      \
	     $(NOTE) "Computing closure" ;			     	      \
	     if $(BUILD_SUPPORT) get_linkage_closure 			      \
	                        $$verboseopt $$ferroropt		      \
	                        -result $$tmpclosurefile		      \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(C_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(C_LINK_OPTIONS)"	      \
	                        -link_mode "$(C_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	     else							      \
	         $(RM) $$tmpclosurefile ;				      \
	         $(NEG) "Unable to compute closure" ;			      \
	         exit 1 ;						      \
	     fi ;							      \
	 else								      \
	     $(NOTE) "Compiling closure" ;				      \
	     if $(BUILD_SUPPORT) get_linkage_closure 			      \
	                        -compile 				      \
	                        $$forceopt $$verboseopt $$ferroropt	      \
	                        -result $$tmpclosurefile	              \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(C_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(C_LINK_OPTIONS)"	      \
	                        -link_mode "$(C_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	     elif [ "$(FIRST_ERROR)" != true -a 			      \
	            "$(ALLOW_CLOSURE_ERROR)" = true ] ; then 		      \
	     	 $(WARN) "Unable to compile closure" ;			      \
	     else							      \
	         $(RM) $$tmpclosurefile ;				      \
	         $(NEG) "Unable to compile closure" ;			      \
	         exit 1 ; 						      \
	     fi ;							      \
	 fi ;								      \
	 if [ "$(EFFORT_ONLY)" != true ] ; then				      \
	 if [ $$altclosurefile != $$closurefile ] ; then		      \
		 $(SED) '/-ptr/d' $$altclosurefile > $$altclosurefile.tmp ;   \
	     $(MV) $$altclosurefile.tmp $$altclosurefile ;		      \
	     fi ;							      \
	 fi ;								      \
	 if $(CMP) -s $$altclosurefile $$closurefile ; then		      \
	     : Both closure files exist and are identical ;		      \
	     copyclosurefile=false ;					      \
	     if [ "$(FORCE)" = "" -a					      \
	          "$(APEX_CHECK_LINK_DEPENDENCIES)" = true -a		      \
	          -f $@ ] ; then					      \
	         uptodate=`$(get_temp) uptodate.$$$$` ;			      \
	         $(ECHO) false > $$uptodate ;				      \
	         $(BUILD_SUPPORT) main_program_is_up_to_date		      \
	                          -result $$uptodate			      \
	                          $@ $$altclosurefile $$closurefile	      \
	                          $(APEX_DIRECTORY)/$@.o ;		      \
	         if [ `cat $$uptodate` = "true" ] ; then		      \
	             $(RM) $$uptodate $$tmpclosurefile;			      \
	             $(UNN) "Main program $@ in $(APEX_DIRECTORY) is up to date" ; \
	             $(POS) "Linking $@" $(SUCCESS_IN_DIR) ;		      \
	             exit 0 ;						      \
	         fi ;							      \
	         $(RM) $$uptodate ;					      \
	     fi ;							      \
	 else								      \
	     : Closure files are different or do not exist;		      \
	     copyclosurefile=true ;					      \
	 fi ;								      \
	 if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(RM) $$tmpclosurefile ;					      \
	     $(EFFORT) "Needs linking: $@ in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ;								      \
	 if [ "$$copyclosurefile" = "true" ] ; then			      \
	     $(CP) $$tmpclosurefile $$closurefile ;			      \
	 fi ;								      \
	 $(RM) $$tmpclosurefile ;					      \
	 $(NOTE) "Starting link" ;					      \
	 if [ "$(APEX_LINK_CLOSURE_LANGUAGES)" = "Ada" ] ; then		      \
	     CMD="$(Rld) $$closurefile $@ $(APEX_VIEW) $(APEX_DIRECTORY) $(APEX_CPP_LINK_MODE)" ; \
	 else								      \
	     CMD="$(ld) $$closurefile $@ $(PURE_ENABLEMENT) $(APEX_ENABLE_PRODUCT)";\
	 fi ;								      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; 							      \
	 if $$CMD ; then						      \
	     $(POS) "Linking $@" $(SUCCESS_IN_DIR) ;			      \
	 else								      \
	     $(NEG) "Linking $@" $(FAILURE_IN_DIR) ;			      \
	     $(TOUCH) $$closurefile ;					      \
	     exit 1 ;							      \
	 fi

$(APEX_CPP_EXECUTABLES) DUMMY2: ALWAYS_OBSOLETE
	@if [ "$(EFFORT_ONLY)" = true ] ; then				      \
	     $(NOTE) "Effort required to link $@ in $(APEX_DIRECTORY)" ;      \
	 else								      \
	     $(NOTE) "Linking $@ in $(APEX_DIRECTORY)" ;		      \
	 fi ;								      \
	 if [ "$(EFFORT_ONLY)" != true -a ! -f $@.o ] ; then 		      \
	     $(NEG) "Unable to find main program object file $@.o" ; exit 1 ; \
	 fi ;								      \
	 if [ "$(APEX_CONFIGURATION)" != "" -a				      \
	      "$(APEX_LINK_CLOSURE_POLICY)" != "imports" ] ; then	      \
	     $(NOTE) "Linking with configuration $(APEX_CONFIGURATION)" ;     \
	 fi ;								      \
	 if [ "$(APEX_CPP_COMPILER_KIND)" != "HP-3" -o                         \
	      "$(APEX_CPP_COMPILER_VARIANT)" != "aCC" ] ; then                \
	     CPP_LINK_PRE_OPTIONS="$(CPP_OPTIONS) $(CPP_LINK_PRE_OPTIONS)" ;  \
	 fi ;								      \
	 closurefile=$(APEX_DIRECTORY)/$@.closure ;			      \
	 tmpclosurefile=`$(get_temp) $@.closure.$$$$` ;			      \
	 altclosurefile=$$tmpclosurefile ;				      \
	 if [ "$(VERBOSE)" = true ] ; then verboseopt="-verbose" ; fi ;       \
	 if [ "$(FORCE)" != "" ] ; then forceopt="-force" ; fi ;              \
	 if [ "$(FIRST_ERROR)" = true ] ; then ferroropt="-first_error" ; fi ;\
	 if [ "$(EFFORT_ONLY)" = true ] ; then				      \
	     if [ "$(COMPILE_CLOSURE)" = true ] ; then			      \
	         $(NOTE) "Computing effort to compile closure" ;	      \
	         if $(BUILD_SUPPORT) get_linkage_closure 		      \
	                        -compile -effort_only			      \
	                        $$forceopt $$verboseopt $$ferroropt	      \
	                        -result $$tmpclosurefile	              \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(CPP_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(CPP_LINK_OPTIONS)"	      \
	                        -link_mode "$(CPP_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	         else							      \
	             $(RM) $$tmpclosurefile ;				      \
	             $(NEG) "Unable to compute closure effort" ;	      \
	             exit 1 ;						      \
	         fi ;							      \
	     fi ;							      \
	 elif [ "$(USE_EXISTING_CLOSURE)" = true ] ; then		      \
	     $(NOTE) "Using existing closure" ;			     	      \
	     if [ -f $$closurefile ] ; then				      \
	         altclosurefile=$$closurefile ;				      \
	     fi ;							      \
	     if $(BUILD_SUPPORT) get_linkage_closure 			      \
	                        $$verboseopt $$ferroropt -use_existing_closure\
	                        -result $$altclosurefile		      \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(CPP_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(CPP_LINK_OPTIONS)"	      \
	                        -link_mode "$(CPP_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	     else							      \
	         $(RM) $$tmpclosurefile ;				      \
	         $(NEG) "Unable to compute closure" ;			      \
	         exit 1 ;						      \
	     fi ;							      \
	 elif [ "$(COMPILE_CLOSURE)" = false ] ; then			      \
	     $(NOTE) "Computing closure" ;			     	      \
	     if $(BUILD_SUPPORT) get_linkage_closure 			      \
	                        $$verboseopt $$ferroropt		      \
	                        -result $$tmpclosurefile		      \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(CPP_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(CPP_LINK_OPTIONS)"	      \
	                        -link_mode "$(CPP_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	     else							      \
	         $(RM) $$tmpclosurefile ;				      \
	         $(NEG) "Unable to compute closure" ;			      \
	         exit 1 ;						      \
	     fi ;							      \
	 else								      \
	     $(NOTE) "Compiling closure" ;				      \
	     if $(BUILD_SUPPORT) get_linkage_closure 			      \
	                        -compile 				      \
	                        $$forceopt $$verboseopt $$ferroropt	      \
	                        -result $$tmpclosurefile	              \
	                        -config "$(APEX_CONFIGURATION)" 	      \
	                        -closure_policy "$(APEX_LINK_CLOSURE_POLICY)" \
	                        -pre_options "$(CPP_LINK_PRE_OPTIONS)"	      \
	                        -post_options "$(CPP_LINK_OPTIONS)"	      \
	                        -link_mode "$(CPP_LINK_MODE)"		      \
	                        $@ ; then : ;				      \
	     elif [ "$(FIRST_ERROR)" != true -a 			      \
	            "$(ALLOW_CLOSURE_ERROR)" = true ] ; then 		      \
	     	 $(WARN) "Unable to compile closure" ;			      \
	     else							      \
	         $(RM) $$tmpclosurefile ;				      \
	         $(NEG) "Unable to compile closure" ;			      \
	         exit 1 ; 						      \
	     fi ;							      \
	 fi ;								      \
	 if $(CMP) -s $$altclosurefile $$closurefile ; then		      \
	     : Both closure files exist and are identical ;		      \
	     copyclosurefile=false ;					      \
	     if [ "$(FORCE)" = "" -a					      \
	          "$(APEX_CHECK_LINK_DEPENDENCIES)" = true -a		      \
	          -f $@ ] ; then					      \
	         uptodate=`$(get_temp) uptodate.$$$$` ;			      \
	         $(ECHO) false > $$uptodate ;				      \
	         $(BUILD_SUPPORT) main_program_is_up_to_date		      \
	                          -result $$uptodate			      \
	                          $@ $$altclosurefile $$closurefile	      \
	                          $(APEX_DIRECTORY)/$@.o ;		      \
	         if [ `cat $$uptodate` = "true" ] ; then		      \
	             $(RM) $$uptodate $$tmpclosurefile;			      \
	             $(UNN) "Main program $@ in $(APEX_DIRECTORY) is up to date" ; \
	             $(POS) "Linking $@" $(SUCCESS_IN_DIR) ;		      \
	             exit 0 ;						      \
	         fi ;							      \
	         $(RM) $$uptodate ;					      \
	     fi ;							      \
	 else								      \
	     : Closure files are different or do not exist;		      \
	     copyclosurefile=true ;					      \
	 fi ;								      \
	 if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(RM) $$tmpclosurefile ;					      \
	     $(EFFORT) "Needs linking: $@ in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ;								      \
	 if [ "$$copyclosurefile" = "true" ] ; then			      \
	     $(CP) $$tmpclosurefile $$closurefile ;			      \
	 fi ;								      \
	 $(RM) $$tmpclosurefile ;					      \
	 $(NOTE) "Starting link" ;					      \
	 if [ "$(APEX_LINK_CLOSURE_LANGUAGES)" = "Ada" ] ; then		      \
	     CMD="$(RLD) $$closurefile $@ $(APEX_VIEW) $(APEX_DIRECTORY) $(APEX_CPP_LINK_MODE)" ; \
	 else								      \
	     CMD="$(LD) $$closurefile $@ $(PURE_ENABLEMENT) $(APEX_ENABLE_PRODUCT)";\
	 fi ;								      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; 							      \
	 if $$CMD ; then						      \
	     $(POS) "Linking $@" $(SUCCESS_IN_DIR) ;			      \
	 else								      \
	     $(NEG) "Linking $@" $(FAILURE_IN_DIR) ;			      \
	     $(TOUCH) $$closurefile ;					      \
	     exit 1 ;							      \
	 fi


.C .cc .cpp .cxx .c:
	@if [ "$(APEX_CPP_COMPILER_KIND)" != "SPARCompiler-5" -a             \
	      \( "$(APEX_CPP_COMPILER_KIND)" != "HP-3" -o                    \
	         "$(APEX_CPP_COMPILER_VARIANT)" != "aCC" \) ] ; then         \
	    $(ERR) "Unable to link $< because it is not a registered main program" ; \
	fi

####

link.files.cpp_component: link.files.announce.cpp_component $(APEX_C_EXECUTABLES) $(APEX_CPP_EXECUTABLES)

link.files.announce.cpp_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Linking C/C++ executables in $(APEX_DIRECTORY)"; 	      \
	 fi

link.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Linking C/C++ executables in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	             $(MAKE) $(STANDARD_OPTION_SETTINGS)		      \
	                     ALLOW_CLOSURE_ERROR=$(ALLOW_CLOSURE_ERROR)	      \
	                     COMPILE_CLOSURE=$(COMPILE_CLOSURE)		      \
	                     USE_EXISTING_CLOSURE=$(USE_EXISTING_CLOSURE)     \
	                     APEX_CONFIGURATION=$(APEX_CONFIGURATION)	      \
	                     link.contents.cpp_component 		      \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

link.contents.code.if_true.no_main_cpp_component:

link.contents.code.if_false.no_main_cpp_component: $(APEX_MAIN_OBJECTS)

link.contents.cpp_component: link.contents.code.if_$(NO_MAIN_PROGRAM_COMPILATION).no_main_cpp_component link.files.cpp_component link.subdirs.cpp_component

##################
# standard targets

object_files.link: link.standard

library.link: link.standard

layer_library.link: link.standard

closure_library.link: link.standard

mutual_library.link: link.standard

mutual_member.link: link.standard

external_library.link: link.noop

ptrepository.link: link.noop

####

link.standard: link.announce_view link.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

link.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Linking view $(APEX_VIEW)" ;   	   		      \
	 fi

link.noop:
	@$(NOTE) "The link operation" $(NO_EFFECT_IN_VIEW)

################################
# cpp component standard targets

link.object_files.cpp_component: link.contents.cpp_component

link.library.cpp_component: link.contents.cpp_component

link.layer_library.cpp_component: link.contents.cpp_component

link.closure_library.cpp_component: link.contents.cpp_component

link.mutual_library.cpp_component: link.contents.cpp_component

link.mutual_member.cpp_component: link.contents.cpp_component

link.external_library.cpp_component:

#################################
# java component standard targets

link.noop.java_component:
	@$(NOTE) "The link operation for Java" $(NO_EFFECT_IN_VIEW)

link.object_files.java_component: link.noop.java_component

link.library.java_component: link.noop.java_component

link.layer_library.java_component: link.noop.java_component

link.closure_library.java_component: link.noop.java_component

link.mutual_library.java_component: link.noop.java_component

link.mutual_member.java_component: link.noop.java_component

link.external_library.java_component: link.noop.java_component

##############################################
# combined java/cpp component standard targets

link.object_files.java_cpp_component: link.object_files.cpp_component

link.library.java_cpp_component: link.library.cpp_component

link.layer_library.java_cpp_component: link.layer_library.cpp_component

link.closure_library.java_cpp_component: link.closure_library.cpp_component

link.mutual_library.java_cpp_component: link.mutual_library.cpp_component

link.mutual_member.java_cpp_component: link.mutual_member.cpp_component

link.external_library.java_cpp_component:

###############################################################################
###############################################################################
# CHECK_RULES OPERATION

.c.check_source .C.check_source .cc.check_source .cpp.check_source .cxx.check_source:
	@$(NOTE) "Checking rules for $< in $(APEX_DIRECTORY)" ;		      \
	 CMD="$(check_rules) $(APEX_DIRECTORY)/$<" ;			      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; $$CMD

.h.check_header .H.check_header .hh.check_header .hpp.check_header .hxx.check_header:
	@$(NOTE) "Checking rules for $< in $(APEX_DIRECTORY)" ;		      \
	 CMD="$(check_rules) $(APEX_DIRECTORY)/$<" ;			      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; $$CMD

check_rules.files.cpp_component: check_rules.files.announce.cpp_component
	@if [ "$(APEX_CPP_SOURCES)$(APEX_C_SOURCES)$(APEX_HEADERS)" = "" ] ; then exit 0 ; fi ; \
	 CMD="$(check_rules) $(APEX_CPP_SOURCES) $(APEX_C_SOURCES) $(APEX_HEADERS)" ; \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $(UMASK) ; $$CMD

check_rules.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ -n "$$subdirs" ] ; then					      \
	    $(NOTE) "Checking rules in subdirectories of $(APEX_DIRECTORY)" ;  \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE) $(STANDARD_OPTION_SETTINGS) check_rules.contents.cpp_component \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

check_rules.contents.cpp_component: check_rules.files.cpp_component check_rules.subdirs.cpp_component

check_rules.files.announce.cpp_component:
	@$(NOTE) "Checking rules for all C/C++ files in $(APEX_DIRECTORY)"

##################
# standard targets

object_files.check_rules: check_rules.standard

library.check_rules: check_rules.standard

layer_library.check_rules: check_rules.standard

closure_library.check_rules: check_rules.standard

mutual_library.check_rules: check_rules.standard

mutual_member.check_rules: check_rules.standard

external_library.check_rules: check_rules.noop

ptrepository.check_rules: check_rules.noop

####

check_rules.standard: check_rules.announce_view check_rules.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

check_rules.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Checking rules for view $(APEX_VIEW)" ;   	      \
	 fi

check_rules.noop:
	@$(NOTE) "The check_rules operation" $(NO_EFFECT_IN_VIEW)

################################
# cpp component standard targets

check_rules.object_files.cpp_component: check_rules.contents.cpp_component

check_rules.library.cpp_component: check_rules.contents.cpp_component

check_rules.layer_library.cpp_component: check_rules.contents.cpp_component

check_rules.closure_library.cpp_component: check_rules.contents.cpp_component

check_rules.mutual_library.cpp_component: check_rules.contents.cpp_component

check_rules.mutual_member.cpp_component: check_rules.contents.cpp_component

check_rules.external_library.cpp_component: check_rules.noop

#################################
# java component standard targets

check_rules.noop.java_component:
	@$(NOTE) "The check_rules operation for Java" $(NO_EFFECT_IN_VIEW)

check_rules.object_files.java_component: check_rules.noop.java_component

check_rules.library.java_component: check_rules.noop.java_component

check_rules.layer_library.java_component: check_rules.noop.java_component

check_rules.closure_library.java_component: check_rules.noop.java_component

check_rules.mutual_library.java_component: check_rules.noop.java_component

check_rules.mutual_member.java_component: check_rules.noop.java_component

check_rules.external_library.java_component:

##############################################
# combined java/cpp component standard targets

check_rules.object_files.java_cpp_component: check_rules.object_files.cpp_component

check_rules.library.java_cpp_component: check_rules.library.cpp_component

check_rules.layer_library.java_cpp_component: check_rules.layer_library.cpp_component

check_rules.closure_library.java_cpp_component: check_rules.closure_library.cpp_component

check_rules.mutual_library.java_cpp_component: check_rules.mutual_library.cpp_component

check_rules.mutual_member.java_cpp_component: check_rules.mutual_member.cpp_component

check_rules.external_library.java_cpp_component:

###############################################################################
###############################################################################
# TEST OPERATION

test.noop:
	@$(NOTE) "The test operation" $(NO_EFFECT_IN_VIEW)

object_files.test: test.noop

library.test: test.noop

layer_library.test: test.noop

closure_library.test: test.noop

mutual_library.test: test.noop

mutual_member.test: test.noop

external_library.test: test.noop

ptrepository.test: test.noop

###############################################################################
###############################################################################
# GENERATE OPERATION

.class.gen_code:
	@if [ "$(EFFORT_ONLY)" = true ] ; then                                \
	     $(EFFORT) "Needs native code generation: $*.java in $(APEX_DIRECTORY)" ; \
	     exit 0 ;                                                         \
	 fi ;                                                                 \
	 $(NOTE) "Generating native code from $*.java in $(APEX_DIRECTORY)" ; \
	 CMD="$(JAVAH) $(JAVA_CLASSPATH) $(APEX_DIRECTORY)/$*.java" ;	      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(UMASK) ;                                                           \
	 if $$CMD ; then                                                      \
	     $(POS) "Generating native code from $*.java" $(SUCCESS_IN_DIR) ; \
	 else                                                                 \
	     $(NEG) "Generating native code from $*.java" $(FAILURE_IN_DIR) ; \
	     exit 1 ;                                                         \
	 fi

####
.java.gen_doc:
	@if [ "$(EFFORT_ONLY)" = true ] ; then                                \
	     $(EFFORT) "Needs document generation: $*.java in $(APEX_DIRECTORY)" ; \
	     exit 0 ;                                                         \
	 fi ;                                                                 \
	 $(NOTE) "Generating documentation from $*.java in $(APEX_DIRECTORY)";\
	 CMD="$(JAVADOC) $(JAVA_CLASSPATH) $(APEX_JAVADOC_OPTIONS) $(APEX_DIRECTORY)/$*.java" ; \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(UMASK) ;                                                           \
	 if $$CMD ; then                                                      \
	     $(POS) "Generating documentation from $*.java" $(SUCCESS_IN_DIR);\
	 else                                                                 \
	     $(NEG) "Generating documentation from $*.java" $(FAILURE_IN_DIR);\
	     exit 1 ;                                                         \
	 fi

generate.files.announce.java_component.generate_code:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Generating native code from Java files in $(APEX_DIRECTORY)" ; \
	 fi

generate.files.announce.java_component.generate_document:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Generating documentation from Java files in $(APEX_DIRECTORY)" ; \
	 fi

generate.files.java_component.generate_code: generate.files.announce.java_component.generate_code $(APEX_JAVA_GENERATE_CODE)

generate.files.java_component.generate_document: generate.files.announce.java_component.generate_document
	@if [ "$(EFFORT_ONLY)" = true ] ; then                                \
	     $(EFFORT) "Needs document generation in $(APEX_DIRECTORY)" ;     \
	     exit 0 ;                                                         \
	 fi ;                                                                 \
	 $(NOTE) "Generating documentation in $(APEX_DIRECTORY)";	      \
	 CMD="$(JAVADOC) $(JAVA_CLASSPATH) $(APEX_JAVADOC_OPTIONS) $(APEX_JAVA_PACKAGE)" ; \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(UMASK) ;                                                           \
	 if $$CMD ; then                                                      \
	     $(POS) "Generating documentation" $(SUCCESS_IN_DIR);	      \
	 else                                                                 \
	     $(NEG) "Generating documentation" $(FAILURE_IN_DIR);	      \
	     exit 1 ;                                                         \
	 fi

generate.files.java_component.generate_all: generate.files.java_component.generate_code	generate.files.java_component.generate_document

generate.files.java_component: generate.files.java_component.generate_$(APEX_GENERATION_TARGET)

generate.subdirs.java_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	             $(MAKE) $(STANDARD_OPTION_SETTINGS)		      \
	                     APEX_GENERATION_TARGET=$(APEX_GENERATION_TARGET) \
	                     generate.contents.java_component		      \
		    ) ; then	: ;			 		      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

generate.contents.java_component: generate.files.java_component generate.subdirs.java_component

##################
# standard targets

object_files.generate: generate.standard

library.generate: generate.standard

layer_library.generate: generate.standard

closure_library.generate: generate.standard

mutual_library.generate: generate.standard

mutual_member.generate: generate.standard

external_library.generate: generate.noop

ptrepository.generate: generate.noop

####

generate.standard: generate.announce_view generate.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

generate.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Generating from view $(APEX_VIEW)" ;   	   	      \
	 fi

generate.noop:
	@$(NOTE) "The generate operation" $(NO_EFFECT_IN_VIEW)

################################
# cpp component standard targets

generate.noop.cpp_component:
	@$(NOTE) "The generate operation for C/C++" $(NO_EFFECT_IN_VIEW)

generate.object_files.cpp_component: generate.noop.cpp_component

generate.library.cpp_component: generate.noop.cpp_component

generate.layer_library.cpp_component: generate.noop.cpp_component

generate.closure_library.cpp_component: generate.noop.cpp_component

generate.mutual_library.cpp_component: generate.noop.cpp_component

generate.mutual_member.cpp_component: generate.noop.cpp_component

generate.external_library.cpp_component: generate.noop.cpp_component

#################################
# java component standard targets

generate.object_files.java_component: generate.contents.java_component

generate.library.java_component: generate.contents.java_component

generate.layer_library.java_component: generate.contents.java_component

generate.closure_library.java_component: generate.contents.java_component

generate.mutual_library.java_component: generate.contents.java_component

generate.mutual_member.java_component: generate.contents.java_component

generate.external_library.java_component:

##############################################
# combined java/cpp component standard targets

generate.object_files.java_cpp_component: generate.object_files.java_component

generate.library.java_cpp_component: generate.library.java_component

generate.layer_library.java_cpp_component: generate.layer_library.java_component

generate.closure_library.java_cpp_component: generate.closure_library.java_component

generate.mutual_library.java_cpp_component: generate.mutual_library.java_component

generate.mutual_member.java_cpp_component: generate.mutual_member.java_component

generate.external_library.java_cpp_component:

###############################################################################
###############################################################################
# DELIVER OPERATION
#
# Only defined for entire views.
# Supports up to 10 different combinations of objects to deliver to
# destination directories.

deliver.contents:
	@if [ "$(APEX_DELIVER_OBJECTS)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY)" ;  \
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS) $(APEX_DELIVER_TO_DIRECTORY)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_1)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_1)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_1)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_1) $(APEX_DELIVER_TO_DIRECTORY_1)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_1) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_1) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_2)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_2)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_2)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_2) $(APEX_DELIVER_TO_DIRECTORY_2)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_2) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_2) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_3)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_3)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_3)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_3) $(APEX_DELIVER_TO_DIRECTORY_3)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_3) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_3) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_4)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_4)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_4)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_4) $(APEX_DELIVER_TO_DIRECTORY_4)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_4) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_4) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_5)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_5)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_5)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_5) $(APEX_DELIVER_TO_DIRECTORY_5)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_5) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_5) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_6)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_6)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_6)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_6) $(APEX_DELIVER_TO_DIRECTORY_6)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_6) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_6) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_7)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_7)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_7)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_7) $(APEX_DELIVER_TO_DIRECTORY_7)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_7) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_7) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_8)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_8)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_8)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_8) $(APEX_DELIVER_TO_DIRECTORY_8)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_8) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_8) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi ;								       \
	 if [ "$(APEX_DELIVER_OBJECTS_9)" != "" -a			       \
	      "$(APEX_DELIVER_TO_DIRECTORY_9)" != "" ] ; then		       \
	     $(NOTE) "Delivering to directory $(APEX_DELIVER_TO_DIRECTORY_9)" ;\
	     CMD="$(COPY) $(APEX_DELIVER_OBJECTS_9) $(APEX_DELIVER_TO_DIRECTORY_9)"; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	       \
	     if $$CMD ; then 						       \
	         $(POS) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_9) $(SUCCESS_FROM_DIR)"; \
	     else							       \
	         $(NEG) "Delivering to $(APEX_DELIVER_TO_DIRECTORY_9) $(FAILURE_FROM_DIR)"; \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ; 	       \
	     fi ;							       \
	 fi

##################
# standard targets

object_files.deliver: deliver.standard

library.deliver: deliver.standard

layer_library.deliver: deliver.standard

closure_library.deliver: deliver.standard

mutual_library.deliver: deliver.standard

mutual_member.deliver: deliver.standard

external_library.deliver: deliver.standard

ptrepository.deliver: deliver.noop

####

deliver.standard: deliver.standard.$(APEX_DIRECTORY_CLASS)

deliver.standard.Ccase_Working: deliver.standard.view

deliver.standard.Working: deliver.standard.view

deliver.standard.Release: deliver.standard.view

deliver.standard.Directory:
	@$(WARN) "Delivering subdirectories is not implemented for $(APEX_DIRECTORY)"

deliver.standard.view: deliver.announce_view deliver.contents

deliver.announce_view:
	@$(NOTE) "Delivering objects from view $(APEX_VIEW)"

deliver.noop:
	@$(NOTE) "The deliver operation" $(NO_EFFECT_IN_VIEW)


###############################################################################
###############################################################################
# FINALIZE OPERATION

finalize.noop:
	@$(NOTE) "The finalize operation" $(NO_EFFECT_IN_VIEW)

object_files.finalize: finalize.noop

library.finalize: finalize.noop

layer_library.finalize: finalize.noop

closure_library.finalize: finalize.noop

mutual_library.finalize: finalize.noop

mutual_member.finalize: finalize.noop

external_library.finalize: finalize.noop

ptrepository.finalize: finalize.noop

###############################################################################
###############################################################################
# DISASSEMBLE OPERATION

###############
# cpp component
# The macro DISASSEMBLE_OBJECTS should contain the simple name of the
# object files to be disassembled

disassemble.objects.cpp_component:
	@if [ "$(DISASSEMBLE_OBJECTS)" != "" ] ; then			      \
	   error=false ;						      \
	   for o in $(DISASSEMBLE_OBJECTS) ; do				      \
	     if [ ! -f $$o ] ; then					      \
	         $(ERR) "Unable to find $$o in $(APEX_DIRECTORY)" ;	      \
	         continue ;						      \
	     fi ;							      \
	     if [ "$(EFFORT_ONLY)" = true ] ; then 			      \
	         $(EFFORT) "Needs disassembling: $$o in $(APEX_DIRECTORY)" ;  \
	         continue ;						      \
	     fi ; 							      \
	     $(NOTE) "Disassembling $$o in $(APEX_DIRECTORY)" ;               \
	     CMD="$(das) $(APEX_DISASSEMBLE_OPTIONS) $$o" ;                   \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;          \
	     $(UMASK) ; 						      \
	     if $$CMD ; then						      \
	         $(POS) "Disassembling $$o" $(SUCCESS_IN_DIR) ;		      \
	     else							      \
	         $(NEG) "Disassembling $$o" $(FAILURE_IN_DIR) ;		      \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ;	      \
	         error=true ;						      \
	     fi ;							      \
	   done ;							      \
	   if [ $$error = true ] ; then exit 1 ; fi ;			      \
	 fi

disassemble.files.cpp_component: disassemble.files.announce.cpp_component
	@found=false ;							      \
	 for f in *.o ; do						      \
	     if [ $$f != "*.o" ] ; then found=true; fi ;		      \
	     objects="$$objects $$f" ;					      \
	 done ;								      \
	 if [ "$$found" = "true" ] ; then				      \
	     $(MAKE) $(STANDARD_OPTION_SETTINGS)				      \
	     	     DISASSEMBLE_OBJECTS="$$objects"			      \
	     	     disassemble.objects.cpp_component ;		      \
	 fi

disassemble.files.announce.cpp_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Disassembling object files in $(APEX_DIRECTORY)" ;	      \
	 fi

disassemble.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Disassembling object files in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	             $(MAKE) $(STANDARD_OPTION_SETTINGS)		      \
	                     disassemble.contents.cpp_component		      \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

disassemble.contents.cpp_component: disassemble.files.cpp_component disassemble.subdirs.cpp_component

################
# java component
#
# The macro DISASSEMBLE_CLASSES should contain the simple name of the classes
# to be disassembled

disassemble.classes.java_component:
	@if [ "$(DISASSEMBLE_CLASSES)" != "" ] ; then			      \
	   error=false ;						      \
	   for c in $(DISASSEMBLE_CLASSES) ; do				      \
	     if [ ! -f $$c ] ; then					      \
	         $(ERR) "Unable to find $$c in $(APEX_DIRECTORY)" ;	      \
	         continue ;						      \
	     fi ;							      \
	     if [ "$(EFFORT_ONLY)" = true ] ; then 			      \
	         $(EFFORT) "Needs disassembling: $$c in $(APEX_DIRECTORY)" ;  \
	         continue ;						      \
	     fi ; 							      \
	     $(NOTE) "Disassembling $$c in $(APEX_DIRECTORY)" ;		      \
	     CMD="$(JAVAP) $(JAVA_CLASSPATH) $(APEX_DISASSEMBLE_OPTIONS) $$c" ; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;          \
	     $(UMASK) ; 						      \
	     if $$CMD ; then						      \
	         $(POS) "Disassembling $$c" $(SUCCESS_IN_DIR) ;		      \
	     else							      \
	         $(NEG) "Disassembling $$c" $(FAILURE_IN_DIR) ;	              \
	         if [ "$(FIRST_ERROR)" = "true" ] ; then exit 1 ; fi ;	      \
	         error=true ;						      \
	     fi ;							      \
	   done ;							      \
	   if [ $$error = true ] ; then exit 1 ; fi ;			      \
	 fi

disassemble.files.java_component: disassemble.files.announce.java_component
	@found=false ;							      \
	 for f in *.java ; do						      \
	     if [ $$f != "*.java" ] ; then 				      \
	         base=`basename $$f .java` ;				      \
	         if [ -f $$base.class ] ; then				      \
	             found="true" ;					      \
	             classes="$$classes $$f" ;				      \
	         fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ "$$found" = "true" ] ; then				      \
	     $(MAKE) $(STANDARD_OPTION_SETTINGS)			      \
	     	     DISASSEMBLE_CLASSES="$$classes"			      \
	     	     disassemble.classes.java_component ;		      \
	 fi

disassemble.files.announce.java_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Disassembling Java classes in $(APEX_DIRECTORY)" ;	      \
	 fi

disassemble.subdirs.java_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ "$(VERBOSE)" = true -a -n "$$subdirs" ] ; then		      \
	    $(NOTE) "Disassembling Java classes in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	             $(MAKE) $(STANDARD_OPTION_SETTINGS)		      \
	                     disassemble.contents.java_component	      \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

disassemble.contents.java_component: disassemble.files.java_component disassemble.subdirs.java_component

##################
# standard targets

object_files.disassemble: disassemble.standard

library.disassemble: disassemble.standard

layer_library.disassemble: disassemble.standard

closure_library.disassemble: disassemble.standard

mutual_library.disassemble: disassemble.standard

mutual_member.disassemble: disassemble.standard

external_library.disassemble: disassemble.noop

ptrepository.disassemble: disassemble.noop

####

disassemble.standard: disassemble.announce_view disassemble.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

disassemble.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Disassembling files in view $(APEX_VIEW)" ; 	      \
	 fi

disassemble.noop:
	@$(NOTE) "The disassemble operation" $(NO_EFFECT_IN_VIEW)


################################
# cpp component standard targets

disassemble.object_files.cpp_component: disassemble.contents.cpp_component

disassemble.library.cpp_component: disassemble.contents.cpp_component

disassemble.layer_library.cpp_component: disassemble.contents.cpp_component

disassemble.closure_library.cpp_component: disassemble.contents.cpp_component

disassemble.mutual_library.cpp_component: disassemble.contents.cpp_component

disassemble.mutual_member.cpp_component: disassemble.contents.cpp_component

disassemble.external_library.cpp_component:

#################################
# java component standard targets

disassemble.object_files.java_component: disassemble.contents.java_component

disassemble.library.java_component: disassemble.contents.java_component

disassemble.layer_library.java_component: disassemble.contents.java_component

disassemble.closure_library.java_component: disassemble.contents.java_component

disassemble.mutual_library.java_component: disassemble.contents.java_component

disassemble.mutual_member.java_component: disassemble.contents.java_component

disassemble.external_library.java_component: disassemble.contents.java_component

##############################################
# combined java/cpp component standard targets

disassemble.object_files.java_cpp_component: disassemble.object_files.java_component disassemble.object_files.cpp_component

disassemble.library.java_cpp_component: disassemble.library.java_component disassemble.library.cpp_component

disassemble.layer_library.java_cpp_component: disassemble.layer_library.java_component disassemble.layer_library.cpp_component

disassemble.closure_library.java_cpp_component: disassemble.closure_library.java_component disassemble.closure_library.cpp_component

disassemble.mutual_library.java_cpp_component: disassemble.mutual_library.java_component disassemble.mutual_library.cpp_component

disassemble.mutual_member.java_cpp_component: disassemble.mutual_member.java_component disassemble.mutual_member.cpp_component

disassemble.external_library.java_cpp_component: disassemble.noop

###############################################################################
###############################################################################
# CLEAN OPERATION

###############
# cpp component

.c.clean .s.clean .C.clean .cc.clean .cpp.clean .cxx.clean:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs cleaning: $< in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 $(NOTE) "Cleaning $< in $(APEX_DIRECTORY)" ;                         \
	 error=false ;							      \
	 CMD="$(clean) $< $(APEX_CLEAN_ARTIFACTS)" ;                          \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 if $$CMD ; then : else error=true ; fi ;                             \
	 CMD="$(RM) $*.o $*.das .Makefiles.rtnl/$*.dep" ;		      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 if $$CMD ; then : ; else error=true ; fi ;                           \
	 CMD="$(TOUCH) .Makefiles.rtnl/$*.dep" ;			      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ; 	      \
	 if $$CMD ; then : ; else error=true ; fi ;			      \
	 if [ $$error = false ] ; then					      \
	     $(POS) "Cleaning $<" $(SUCCESS_IN_DIR) ;			      \
	 else								      \
	     $(NEG) "Cleaning $<" $(FAILURE_IN_DIR) ;		              \
	     exit 1 ;							      \
	 fi

clean.deleted.cpp_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	  $(NOTE) "Cleaning $(CLEAN_ROOT).$(CLEAN_EXT) in $(APEX_DIRECTORY)" ;\
	 fi ;								      \
	 CMD="$(clean) $(CLEAN_ROOT).$(CLEAN_EXT) $(APEX_CLEAN_ARTIFACTS)" ;  \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $$CMD ;  		                                              \
	 CMD="$(RM) $(CLEAN_ROOT).o .Makefiles.rtnl/$(CLEAN_ROOT).dep" ;      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $$CMD

CUSTOM_CLEAN_CPP_OBJECTS = $(APEX_CLEAN_C_SOURCES) $(APEX_CLEAN_CPP_SOURCES) $(APEX_CLEAN_HEADERS)

ADD_IN_CLEAN_CPP_OBJECTS = $(ADD_IN_CLEAN_CPP) $(ADD_IN_CLEAN_CPP_1) $(ADD_IN_CLEAN_CPP_2) $(ADD_IN_CLEAN_CPP_3) $(ADD_IN_CLEAN_CPP_4) $(ADD_IN_CLEAN_CPP_5) $(ADD_IN_CLEAN_CPP_6) $(ADD_IN_CLEAN_CPP_7) $(ADD_IN_CLEAN_CPP_8) $(ADD_IN_CLEAN_CPP_9)

clean.files.cpp_component:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs C/C++ cleaning: $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 $(NOTE) "Cleaning C/C++ artifacts in $(APEX_DIRECTORY)" ;            \
	 error=false ;							      \
	 CMD="$(BUILD_SUPPORT) delete -force -dir . \"*.o\" \"*.das\" \"*.a\" \"*.so\" \"*.sl\" \"*.a.closure\" \"*.so.closure\" \"*.sl.closure\" " ; \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(BUILD_SUPPORT) delete -force -dir . "*.o" "*.das" "*.a" "*.so" "*.sl" "*.a.closure" "*.so.closure" "*.sl.closure" ; \
	 if [ $$? -eq 0 ] ; then : ; else error=true ; fi ;                   \
	 $(UMASK) ;							      \
	 CMD="$(RM) \".Makefiles.rtnl/*.dep\" ; $(TOUCH) \".Makefiles.rtnl/*.dep\"" ; \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 for i in .Makefiles.rtnl/*.dep ; do 				      \
	     if [ "$$i" != ".Makefiles.rtnl/*.dep" ] ; then		      \
	         CMD="$(RM) $$i" ; 					      \
	         if $$CMD ; then : ; else error=true ; fi ;		      \
	         CMD="$(TOUCH) $$i" ;					      \
	         if $$CMD ; then : ; else error=true ; fi ;		      \
	     fi ;						 	      \
	 done ;								      \
         if [ "$(APEX_PTREPOSITORY_DIRECTORY)" = "" ] ; then		      \
	     ptr_dir="nil" ;						      \
	 else								      \
	     ptr_dir="$(APEX_PTREPOSITORY_DIRECTORY)" ;			      \
	 fi ;								      \
	 if [ "$(APEX_PTREPOSITORY_OPTION)" = "" ] ; then		      \
	     ptr_params="nil" ;				      \
	 else								      \
	     ptr_params="$(APEX_PTREPOSITORY_OPTION)" ;	      \
	 fi ;								      \
	 CMD="$(clean) $$ptr_dir -dir $(APEX_DIRECTORY) $(APEX_CLEAN_ARTIFACTS)" ;\
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 if $$CMD ; then : ; else error=true ; fi ;                           \
	 executables="$(APEX_C_EXECUTABLES)" ;				      \
	 for i in $$executables ; do 					      \
	     CMD="$(RM) $$i $$i.closure" ;				      \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;          \
	     if $$CMD ; then : ; else error=true ; fi ;                       \
	 done ;								      \
	 executables="$(APEX_CPP_EXECUTABLES)" ;			      \
	 for i in $$executables ; do 					      \
	     CMD="$(RM) $$i $$i.closure" ;				      \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;          \
	     if $$CMD ; then : ; else error=true ; fi ;                       \
	 done ;								      \
	 for i in $(CUSTOM_CLEAN_CPP_OBJECTS) $(ADD_IN_CLEAN_CPP_OBJECTS) nil ; do \
	     if [ "$$i" != "nil" ] ; then				      \
	         CMD="$(RM) $$i" ; 					      \
	         if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;      \
	         if $$CMD ; then : ; else error=true ; fi ;		      \
	     fi ;							      \
	 done ;								      \
	 if [ $$error = false ] ; then					      \
	     $(POS) "Cleaning C/C++ artifacts" $(SUCCESS_IN_DIR) ;	      \
	 else								      \
	     $(NEG) "Cleaning C/C++ artifacts" $(FAILURE_IN_DIR) ;	      \
	     exit 1 ;							      \
	 fi

clean.subdirs.cpp_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ -n "$$subdirs" ] ; then					      \
	    $(NOTE) "Cleaning C/C++ artifacts in subdirectories of $(APEX_DIRECTORY)" ; \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE2) -f $(ROOT_MAKEFILE) 			      \
		              ROOT_MAKEFILE=$(ROOT_MAKEFILE) 		      \
	                      $(STANDARD_OPTION_SETTINGS) 		      \
	                      clean.contents.cpp_component		      \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

clean.contents.cpp_component: clean.files.cpp_component clean.subdirs.cpp_component

clean.contents.cpp_component.pt:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs C++ cleaning: $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 $(NOTE) "Cleaning C++ template repository artifacts in $(APEX_DIRECTORY)" ; \
	 error=false ;							      \
         if [ "$(APEX_PTREPOSITORY_DIRECTORY)" = "" ] ; then		      \
	     ptr_dir="nil" ;						      \
	 else								      \
	     ptr_dir="$(APEX_PTREPOSITORY_DIRECTORY)" ;			      \
	 fi ;								      \
	 if [ "$(APEX_PTREPOSITORY_OPTION)" = "" ] ; then		      \
	     ptr_params="nil" ;						      \
	 else								      \
	     ptr_params="$(APEX_PTREPOSITORY_OPTION)" ;		      \
	 fi ;								      \
	 CMD="$(clean) $$ptr_dir -dir $(APEX_DIRECTORY) $(APEX_CLEAN_ARTIFACTS)" ;\
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 if $$CMD ; then : ; else error=true ; fi ;                           \
	 if [ $$error = false ] ; then					      \
	     $(POS) "Cleaning C++ template repository artifacts" $(SUCCESS_IN_DIR) ;			      \
	 else								      \
	     $(NEG) "Cleaning C++ template repository artifacts" $(FAILURE_IN_DIR) ;		              \
	     exit 1 ;							      \
	 fi

################
# java component

.java.clean:
	@if [ "$(EFFORT_ONLY)" = true ] ; then                                \
	     $(EFFORT) "Needs cleaning: $< in $(APEX_DIRECTORY)" ;            \
	     exit 0 ;                                                         \
	 fi ;                                                                 \
	 $(NOTE) "Cleaning $< in $(APEX_DIRECTORY)" ;                         \
	 CMD="$(RM) $*.class $*.jdas" ;                                       \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 if $$CMD ; then						      \
	     $(POS) "Cleaning $<" $(SUCCESS_IN_DIR) ;			      \
	 else								      \
	     $(NEG) "Cleaning $<" $(FAILURE_IN_DIR) ;		              \
	     exit 1 ;							      \
	 fi

clean.deleted.java_component:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	     $(NOTE) "Cleaning $(CLEAN_ROOT).$(CLEAN_EXT) in $(APEX_DIRECTORY)" ; \
	 fi ; 								      \
	 CMD="$(RM) $(CLEAN_ROOT).class" ;				      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $$CMD

CUSTOM_CLEAN_JAVA_OBJECTS = $(APEX_CLEAN_JAVA_SOURCES)

clean.files.java_component:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs Java cleaning: $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 $(NOTE) "Cleaning Java artifacts in $(APEX_DIRECTORY)";              \
	 CMD="$(BUILD_SUPPORT) delete -force -dir . \"*.class\" \"*.jdas\" " ;\
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(BUILD_SUPPORT) delete -force -dir . "*.class" "*.jdas" ;	      \
	 for i in $(CUSTOM_CLEAN_JAVA_OBJECTS) nil ; do			      \
	     if [ "$$i" != "nil" ] ; then				      \
	         CMD="$(RM) $$i" ; 					      \
	         if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;      \
	         if $$CMD ; then : ; else error=true ; fi ;		      \
	     fi ;							      \
	 done ;								      \
	 if [ $$? -eq 0 ] ; then 					      \
	     $(POS) "Cleaning Java artifacts" $(SUCCESS_IN_DIR) ;	      \
	 else								      \
	     $(NEG) "Cleaning Java artifacts" $(FAILURE_IN_DIR) ;	      \
	     exit 1 ;							      \
	 fi

clean.subdirs.java_component:
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 subdirerror=false ;						      \
	 if [ -n "$$subdirs" ] ; then					      \
	    $(NOTE) "Cleaning java artifacts in subdirectories of $(APEX_DIRECTORY)" ;  \
	 fi ;								      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         if ( cd $$i ; 						      \
	              $(MAKE2) $(STANDARD_OPTION_SETTINGS) clean.contents.java_component \
		    ) ; then	: ;				 	      \
		 else							      \
		     $(NEG) $$i "processing failed" ;			      \
		     subdirerror=true ;					      \
		 fi ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ $$subdirerror = true ] ; then				      \
	     exit 1 ;							      \
	 fi

clean.contents.java_component: clean.files.java_component clean.subdirs.java_component

clean.noop:
	@$(NOTE) "The clean operation" $(NO_EFFECT_IN_VIEW)

##################
# standard targets

object_files.clean: clean.standard

library.clean: clean.standard

layer_library.clean: clean.standard

closure_library.clean: clean.standard

mutual_library.clean: clean.standard

mutual_member.clean: clean.standard

external_library.clean: clean.noop

ptrepository.clean: clean.standard

####

clean.standard: clean.announce_view clean.$(APEX_BUILD_POLICY).$(APEX_BUILD_COMPONENT)_component

clean.announce_view:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Cleaning view $(APEX_VIEW)" ;   	   		      \
	 fi

################################
# cpp component standard targets

clean.object_files.cpp_component: clean.contents.cpp_component

clean.library.cpp_component: clean.contents.cpp_component

clean.layer_library.cpp_component: clean.contents.cpp_component

clean.closure_library.cpp_component: clean.contents.cpp_component

clean.mutual_library.cpp_component: clean.contents.cpp_component

clean.mutual_member.cpp_component: clean.contents.cpp_component

clean.external_library.cpp_component:

clean.ptrepository.cpp_component: clean.contents.cpp_component.pt

#################################
# java component standard targets

clean.object_files.java_component: clean.contents.java_component

clean.library.java_component: clean.contents.java_component

clean.layer_library.java_component: clean.contents.java_component

clean.closure_library.java_component: clean.contents.java_component

clean.mutual_library.java_component: clean.contents.java_component

clean.mutual_member.java_component: clean.contents.java_component

clean.external_library.java_component:

clean.ptrepository.java_component:

##############################################
# combined java/cpp component standard targets

clean.object_files.java_cpp_component: clean.object_files.java_component clean.object_files.cpp_component

clean.library.java_cpp_component: clean.library.java_component clean.library.cpp_component

clean.layer_library.java_cpp_component: clean.layer_library.java_component clean.layer_library.cpp_component

clean.closure_library.java_cpp_component: clean.closure_library.java_component clean.closure_library.cpp_component

clean.mutual_library.java_cpp_component: clean.mutual_library.java_component clean.mutual_library.cpp_component

clean.mutual_member.java_cpp_component: clean.mutual_member.java_component clean.mutual_member.cpp_component

clean.external_library.java_cpp_component:

clean.ptrepository.java_cpp_component:

###############################################################################
###############################################################################
# BUILD OPERATION
#
# Actual effect of "build" is based on the contents of APEX_BUILD_PHASES
# which is a comma separated list of apex commands and possible options
# for those commands.  For example, "code , link" specifies that "apex code"
# and then "apex link" should be run on the view.

build.contents:
	@cmd_file=`$(get_temp) build_cmd_file_$$$$` ;			      \
	 $(ECHO) "setenv   APEX_CANONICAL_PREFIX true"     > $$cmd_file ;     \
	 $(ECHO) "unsetenv APEX_OUTPUT_WINDOW"            >> $$cmd_file ;     \
	 $(ECHO) "unsetenv APEX_OUTPUT_MESSAGE"           >> $$cmd_file ;     \
	 $(ECHO) "setenv   APEX_OUTPUT_FILTER $(FILTERS)" >> $$cmd_file ;     \
	 $(ECHO) 					  >> $$cmd_file ;     \
	 if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(ECHO) "setenv   APEX_EFFORT_ONLY true"     >> $$cmd_file ;     \
	 else								      \
	     $(ECHO) "unsetenv APEX_EFFORT_ONLY"	  >> $$cmd_file ;     \
	 fi ;								      \
	 if [ "$(FORCE)" != "" ] ; then					      \
	     $(ECHO) "setenv   APEX_FORCE true"		  >> $$cmd_file ;     \
	 else								      \
	     $(ECHO) "unsetenv APEX_FORCE"		  >> $$cmd_file ;     \
	 fi ;								      \
	 if [ "$(VERBOSE)" = true ] ; then				      \
	     $(ECHO) "setenv   APEX_VERBOSE true"	  >> $$cmd_file ;     \
	 else							              \
	     $(ECHO) "unsetenv APEX_VERBOSE"	   	  >> $$cmd_file ;     \
	 fi ;								      \
	 if [ "$(FIRST_ERROR)" = true ] ; then				      \
	     $(ECHO) "setenv   APEX_FIRST_ERROR true"	  >> $$cmd_file ;     \
	 else								      \
	     $(ECHO) "unsetenv APEX_FIRST_ERROR"	  >> $$cmd_file ;     \
	 fi ;								      \
	 if [ "$${APEX_BATCH-unset}" != "unset" ] ; then		      \
	     $(ECHO) "setenv   APEX_BATCH true"		  >> $$cmd_file ;     \
	 else								      \
	     $(ECHO) "unsetenv APEX_BATCH"		  >> $$cmd_file ;     \
	 fi ;								      \
	 $(ECHO) 					  >> $$cmd_file ;     \
	 $(ECHO) '$(APEX_BUILD_PHASES)'			  >> $$cmd_file ;     \
	 CMD="$(APEX_SHELL) $$cmd_file" ;				      \
	 if [ "$(VERBOSE)" = true ] ; then 				      \
	     $(TRACE) $$CMD ;						      \
	     $(NOTE)  "$$cmd_file :" ;					      \
	     $(CAT)   $$cmd_file ;					      \
	 fi ; 								      \
	 if $$CMD ; then 						      \
	     $(RM) $$cmd_file ;						      \
	 elif [ "$(FIRST_ERROR)" = "true" ] ; then 			      \
	     $(RM) $$cmd_file ;						      \
	     exit 1 ; 							      \
	 else								      \
	     $(RM) $$cmd_file ;						      \
	 fi


##################
# standard targets

object_files.build: build.standard

library.build: build.standard

layer_library.build: build.standard

closure_library.build: build.standard

mutual_library.build: build.standard

mutual_member.build: build.standard

external_library.build: build.standard

ptrepository.build: build.noop

####

build.standard: build.standard.$(APEX_DIRECTORY_CLASS)

build.standard.Ccase_Working: build.standard.view

build.standard.Working: build.standard.view

build.standard.Release: build.standard.view

build.standard.Directory:
	@$(WARN) "Building subdirectories is not implemented for $(APEX_DIRECTORY)"

build.standard.view: build.announce_view build.contents

build.announce_view:
	@$(NOTE) "Building view $(APEX_VIEW)"


###############################################################################
###############################################################################
# LINK QUERY
#
# These rules are used to query a view to determine its contribution
# when linking a main program for which the view is in the closure.
#
# These rules are set up to allow recompilation during the same recursive
# calls that are used to compute the contribution.
#
# Link Query only applies to the cpp component.

object_files.link_contribution.query: link_contribution.query_precondition
	@if [ "$(APEX_LINK_CONTRIBUTION_PRE_OPTIONS)" != "" ] ; then 	      \
	     $(ECHO) $(APEX_LINK_CONTRIBUTION_PRE_OPTIONS) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	 fi ;								      \
	 if [ "$(APEX_PTREPOSITORY_OPTION)" != "nil" ] ; then		      \
	     $(ECHO) "$(APEX_PTREPOSITORY_OPTION)" >> $(LINK_QUERY_RESULT) ;  \
	 fi ;								      \
	 objects="$(OBJECTS)" ;						      \
	 for i in $$objects ; do					      \
	     if [ -f $$i ] ; then					      \
	         $(ECHO) $(APEX_DIRECTORY)/$$i >> $(LINK_QUERY_RESULT) ;      \
	     else							      \
	     	 $(ERR) "Unable to find $$i in $(APEX_DIRECTORY)" ;	      \
	     fi ;							      \
	 done ;								      \
	 subdirs="$(APEX_SUBDIRS)" ;					      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         (cd $$i ; 						      \
	          $(MAKE) -f $(ROOT_MAKEFILE)				      \
	                  ROOT_MAKEFILE=$(ROOT_MAKEFILE) 		      \
	                  LINK_QUERY_RESULT=$(LINK_QUERY_RESULT)	      \
	                  $(STANDARD_OPTION_SETTINGS) 			      \
	                  subdir_object_files.link_contribution.query 	      \
	         ) ;							      \
	     fi ;							      \
	 done ;								      \
	 if [ "$(APEX_LINK_CONTRIBUTION_OPTIONS)" != "" ] ; then	      \
	     $(ECHO) $(APEX_LINK_CONTRIBUTION_OPTIONS)			      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	 fi ;								      \
	 if [ "$(GLOBAL_LINK_OBJ_CONTRIB_OPTIONS)" != "" ] ; then	      \
	     $(ECHO) $(GLOBAL_LINK_OBJ_CONTRIB_OPTIONS) >> $(LINK_QUERY_RESULT); \
	 fi

subdir_object_files.link_contribution.query:
	@objects="$(APEX_OBJECTS)" ;					      \
	 for i in $$objects ; do					      \
	     $(ECHO) $(APEX_DIRECTORY)/$$i >> $(LINK_QUERY_RESULT) ;	      \
	 done ;								      \
	 subdirs="$(APEX_SUBDIRS)" ;					      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	         (cd $$i ; 						      \
	          $(MAKE) -f $(ROOT_MAKEFILE)				      \
	                  ROOT_MAKEFILE=$(ROOT_MAKEFILE) 		      \
	                  LINK_QUERY_RESULT=$(LINK_QUERY_RESULT) 	      \
	                  $(STANDARD_OPTION_SETTINGS)			      \
	                  subdir_object_files.link_contribution.query 	      \
	         ) ;							      \
	     fi ;							      \
	 done

# For library policies, check the link mode to see which type of library
# should be used.

standard_library.link_contribution.query: link_contribution.query_precondition
	@if [ "$(LINK_QUERY_MODE)" = "static" ] ; then			      \
	     if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" != true ] ; then          \
	         $(ERR) "Unable to link against archived library in view $(APEX_VIEW) because the CREATE_ARCHIVED_LIBRARY switch is not set" ; \
	         exit 1 ;						      \
	     fi ;							      \
	     if [ ! -f "$(APEX_LIBRARY_NAME)" ] ; then			      \
	         if [ ! -f "$(NO_OBJECTS_FOR_LIB_FLAG)" -a		      \
	              "$(EFFORT_ONLY)" != true ] ; then		      \
	             $(ERR) "Unable to find archived library $(APEX_LIBRARY_NAME) in $(APEX_VIEW)" ; \
	             exit 1 ;						      \
	         fi ; 							      \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_PRE_OPTIONS)" != "" ] ; then      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_PRE_OPTIONS) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	     if [ -f "$(APEX_LIBRARY_NAME)" -o 				      \
	          "$(EFFORT_ONLY)" = true ] ; then			      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_LIBRARY) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_OPTIONS)" != "" ] ; then	      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_OPTIONS)		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	 elif [ "$(LINK_QUERY_MODE)" = "dynamic" ] ; then		      \
	     if [ "$(APEX_CREATE_SHARED_LIBRARY)" != true ] ; then            \
	         $(ERR) "Unable to link against shared library in view $(APEX_VIEW) because the CREATE_SHARED_LIBRARY switch is not set" ; \
		 exit 1 ;						      \
	     fi ;							      \
	     if [ ! -f "$(APEX_SHARED_LIBRARY_NAME)" ] ; then		      \
	         if [ ! -f "$(NO_OBJECTS_FOR_LIB_FLAG)" -a		      \
	              "$(EFFORT_ONLY)" != true ] ; then			      \
	             $(ERR) "Unable to find shared library $(APEX_SHARED_LIBRARY_NAME) in $(APEX_VIEW)" ; \
	             exit 1 ;						      \
	         fi ; 							      \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)" != "" ] ; then \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS) 	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	     if [ -f "$(APEX_SHARED_LIBRARY_NAME)" -o			      \
	          "$(EFFORT_ONLY)" = true ] ; then			      \
	     	 $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_LIBRARY) 	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ; 							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)" != "" ] ; then   \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	 elif [ "$(LINK_QUERY_MODE)" = "dynamic_or_static" ] ; then	      \
	     if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" != true -a		      \
	          "$(APEX_CREATE_SHARED_LIBRARY)" != true ] ; then	      \
	         $(ERR) "Unable to link against library in view $(APEX_VIEW) because neither the CREATE_ARCHIVED_LIBRARY or CREATE_SHARED_LIBRARY switch is set" ; \
		 exit 1 ;						      \
	     fi ;							      \
	     if [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" = true -a		      \
	          -f "$(APEX_LIBRARY_NAME)" ] ; then			      \
	         : ;							      \
	     elif [ "$(APEX_CREATE_SHARED_LIBRARY)" = true -a		      \
	            -f "$(APEX_SHARED_LIBRARY_NAME)" ] ; then		      \
	         : ;							      \
	     elif [ ! -f "$(NO_OBJECTS_FOR_LIB_FLAG)" -a		      \
	            "$(EFFORT_ONLY)" != true ] ; then			      \
	         $(ERR) "Unable to find archived library $(APEX_LIBRARY_NAME) or shared library $(APEX_SHARED_LIBRARY_NAME) in $(APEX_VIEW)" ; \
	         exit 1 ;						      \
	     fi ;							      \
	     if [ "$(APEX_CREATE_SHARED_LIBRARY)" = true -a 		      \
	          \( -f $(APEX_SHARED_LIBRARY_NAME) -o			      \
	            "$(EFFORT_ONLY)" = true \) ] ; then     	              \
	       if [ "$(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)" != "" ] ; then \
	           $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)       \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	       $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_LIBRARY) 	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       if [ "$(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)" != "" ] ; then \
	           $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	     elif [ "$(APEX_CREATE_ARCHIVED_LIBRARY)" = true -a 	      \
	            \( -f "$(APEX_LIBRARY_NAME)" -o			      \
	              "$(EFFORT_ONLY)" = true \) ] ; then		      \
	       if [ "$(APEX_LINK_CONTRIBUTION_PRE_OPTIONS)" != "" ] ; then    \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_PRE_OPTIONS) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	       $(ECHO) $(APEX_LINK_CONTRIBUTION_LIBRARY) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       if [ "$(APEX_LINK_CONTRIBUTION_OPTIONS)" != "" ] ; then	      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_OPTIONS)		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	     elif [ "$(APEX_CREATE_SHARED_LIBRARY)" = true ] ; then	      \
	       if [ "$(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)" != "" ] ; then \
	           $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)       \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	       if [ "$(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)" != "" ] ; then \
	           $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	     else							      \
	       if [ "$(APEX_LINK_CONTRIBUTION_PRE_OPTIONS)" != "" ] ; then    \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_PRE_OPTIONS) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	       if [ "$(APEX_LINK_CONTRIBUTION_OPTIONS)" != "" ] ; then	      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_OPTIONS)		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	       fi ;							      \
	     fi ;							      \
	 else								      \
	     $(ERR) "Unrecognized link mode \"$(LINK_QUERY_MODE)\" encountered while computing link contribution for $(APEX_VIEW)" ; \
	     exit 1 ;							      \
	 fi ;								      \
	 tmp_new_files=`$(get_temp) tmp_files_$$$$` ;		       	      \
	 $(TOUCH) $$tmp_new_files ; 					      \
	 if [ "$(APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS)" != "true" ] ; then \
	     ( $(MAKE) CONTENT_QUERY_RESULT=$$tmp_new_files	              \
		cpp_ptrepository.query ) ;				      \
	 else								      \
	     ( $(MAKE) CONTENT_QUERY_RESULT=$$tmp_new_files	              \
		object_files.cpp_ptrepository.query ) ;			      \
	 fi ;								      \
	 $(CAT) $$tmp_new_files >> $(LINK_QUERY_RESULT) ;		      \
	 $(RM) $$tmp_new_files


# cpp_ptrespository_query for each BUILD_POLICY

library.cpp_ptrepository.query: standard_library.cpp_ptrepository.query

layer_library.cpp_ptrepository.query: standard_library.cpp_ptrepository.query

closure_library.cpp_ptrepository.query: standard_library.cpp_ptrepository.query

layer_library.cpp_ptrepository.query: standard_library.cpp_ptrepository.query

mutual_library.cpp_ptrepository.query: standard_library.cpp_ptrepository.query

mutual_member.cpp_ptrepository.query: object_files.cpp_ptrepository.query

external_library.cpp_ptrepository.query:

ptrepository.cpp_ptrepository.query:

object_files.cpp_ptrepository.query:
	@if [ "$(APEX_CPP_COMPILER_KIND)" != "HP-3" ] ; then		      \
	     (								      \
		$(MAKE) CONTENT_QUERY_RESULT=$(CONTENT_QUERY_RESULT)	      \
			cpp_ptrepository_options.query			      \
		) ;							      \
	elif [ "$(APEX_CPP_COMPILER_VARIANT)" != "aCC" ] ; then               \
	     (								      \
	 	$(MAKE) CONTENT_QUERY_RESULT=$(CONTENT_QUERY_RESULT)	      \
			cpp_include_options.query	;		      \
		$(MAKE) CONTENT_QUERY_RESULT=$(CONTENT_QUERY_RESULT)	      \
                      cpp_ptrepository_options.query			      \
		 ) ;							      \
	fi

standard_library.cpp_ptrepository.query:
	@if [ "$(APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS)" != "true" ] ; then \
	     tmp_new_files=`$(get_temp) tmp_files_$$$$` ;		      \
	     lib_new_views=`$(get_temp) new_views_$$$$` ;		      \
	     $(TOUCH) $$lib_new_views $$tmp_new_files ;			      \
	     case "$(APEX_BUILD_POLICY)" in 				      \
	         library) 						      \
	             $(ECHO) $(APEX_VIEW) >> $$lib_new_views ;		      \
	             ;;							      \
	         layer_library)						      \
	    	     $(BUILD_SUPPORT) get_explicit_imports -result $$lib_new_views -self ; \
	    	     ;;							      \
	    	 closure_library)					      \
	    	     $(BUILD_SUPPORT) get_all_imports -result $$lib_new_views -self ; \
	    	     ;;							      \
	    	 mutual_library)					      \
	   	     $(BUILD_SUPPORT) get_mutual_imports -result $$lib_new_views -self ; \
	   	     ;;							      \
	   	 *)							      \
	   	     ;;							      \
	     esac ;							      \
	     for i in `$(CAT) $$lib_new_views` ; do			      \
		 if [ $$i = "$(APEX_VIEW)" ] ; then			      \
		     (cd $$i ;						      \
		     	$(APEX_MAKE) -all				      \
				CONTENT_QUERY_RESULT=$$tmp_new_files	      \
				object_files.cpp_ptrepository.query ;	      \
			) ;						      \
	         elif [ -d $$i ] ; then					      \
		     (cd $$i ;						      \
		         $(APEX_MAKE) -all				      \
				CONTENT_QUERY_RESULT=$$tmp_new_files	      \
			 	cpp_ptrepository.query	;		      \
		     ) ;						      \
	         else							      \
	             $(ERR) "$(APEX_VIEW): Import $$i does not exist" ;	      \
	         fi ;							      \
	     done ; 				 		              \
	     $(CAT) $$tmp_new_files >> $(CONTENT_QUERY_RESULT) ;	      \
	     $(RM) $$tmp_new_files $$lib_new_views ;			      \
	 fi

library.link_contribution.query: standard_library.link_contribution.query

layer_library.link_contribution.query: standard_library.link_contribution.query

closure_library.link_contribution.query: standard_library.link_contribution.query

mutual_library.link_contribution.query: standard_library.link_contribution.query

mutual_member.link_contribution.query: link_contribution.query_precondition

external_library.link_contribution.query: link_contribution.query_precondition
	@if [ "$(LINK_QUERY_MODE)" = "static" ] ; then			      \
	     if [ "$(APEX_LINK_CONTRIBUTION_PRE_OPTIONS)" != "" ] ; then      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_PRE_OPTIONS) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_OPTIONS)" != "" ] ; then	      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_OPTIONS)		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	 elif [ "$(LINK_QUERY_MODE)" = "dynamic" ] ; then		      \
	     if [ "$(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)" != "" ] ; then \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS) 	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)" != "" ] ; then   \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	 elif [ "$(LINK_QUERY_MODE)" = "dynamic_or_static" ] ; then	      \
	     dynamic=false ;						      \
	     if [ "$(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS)" != "" ] ; then \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_PRE_OPTIONS) 	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	         dynamic=true ;	     					      \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)" != "" ] ; then   \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_SHARED_OPTIONS)	      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	         dynamic=true ;						      \
	     fi ;							      \
	     if [ $$dynamic = true ] ; then exit 0 ; fi ;		      \
	     if [ "$(APEX_LINK_CONTRIBUTION_PRE_OPTIONS)" != "" ] ; then      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_PRE_OPTIONS) 		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	     if [ "$(APEX_LINK_CONTRIBUTION_OPTIONS)" != "" ] ; then	      \
	         $(ECHO) $(APEX_LINK_CONTRIBUTION_OPTIONS)		      \
	     					>> $(LINK_QUERY_RESULT) ;     \
	     fi ;							      \
	 else								      \
	     $(ERR) "Unrecognized link mode \"$(LINK_QUERY_MODE)\" encountered while computing link contribution for $(APEX_VIEW)" ;	      \
	     exit 1 ;							      \
	 fi

ptrepository.link_contribution.query:

###############################################################################
# Linkage Query Preconditions
#
# These rules use the current setting of COMPILE_CLOSURE to determine if
# the closure should be compiled as a precondition to generating the closure
# query.
#
# When compilation is requested these rules apply the normal customization
# macros as part of compiling the view itself.


link_contribution.query_precondition: link_contribution.query_precondition.announce link_contribution.query_precondition_$(COMPILE_CLOSURE)

link_contribution.query_precondition_false:

link_contribution.query_precondition_true:
	@if [ "$(EFFORT_ONLY)" = true ] ; then effortopt="-effort_only" ; fi ;\
	 if [ "$(FORCE)" != "" ] ;        then forceopt="-force" ; fi ;	      \
	 if [ "$(VERBOSE)" = true ] ;     then verboseopt="-verbose" ; fi ;   \
	 if [ "$(FIRST_ERROR)" = true ] ; then ferroropt="-first_error" ; fi ;\
	 nompcopt="-no_main_program_compilation" ;			      \
	 options="$$forceopt $$verboseopt $$effortopt $$ferroropt $$nompcopt" ;	\
	 CMD="$(COMPILE) $$options $(APEX_DIRECTORY)" ;			      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	 $$CMD

link_contribution.query_precondition.announce:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   if [ "$(COMPILE_CLOSURE)" = false ] ; then			      \
	       $(NOTE) "Computing link contribution for view $(APEX_VIEW)" ;  \
	    else							      \
	       $(NOTE) "Coding and computing link contribution for view $(APEX_VIEW)" ; \
	   fi								      \
	 fi

###############################################################################
###############################################################################
# OBJECT FILES CONTENT QUERY
#
# Query a view or directory (possibly recursively) for the object files in the
# view.  The query is based on the build policy which is particularly useful
# for customization.  This is generally used to get the object files that are
# to go into an archive.
#
# Only applies to the cpp component.

simple_object_files_content.query:
	@objects="$(APEX_OBJECTS)" ;					      \
	 for i in $$objects ; do					      \
	     if [ -f $$i ] ; then					      \
	         $(ECHO) $(APEX_DIRECTORY)/$$i >> $(CONTENT_QUERY_RESULT) ;   \
	     else							      \
	     	 $(ERR) "Unable to find $$i in $(APEX_DIRECTORY)" ;	      \
	     fi ;							      \
	 done

subdir_object_files_content.query: simple_object_files_content.query
	@subdirs="$(APEX_SUBDIRS)" ;					      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	       (cd $$i ; 						      \
	        $(MAKE) CONTENT_QUERY_RESULT=$(CONTENT_QUERY_RESULT)	      \
			APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS=$(APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS) \
	                subdir_object_files_content.query		      \
	       ) ;							      \
	     fi ;							      \
	 done

view_object_files_content.query:
	@$(NOTE) "Computing object files in $(APEX_VIEW)" ;		      \
	 regular_objs_content=`$(get_temp) reg_objs_content_$$$$` ;	      \
	 template_objs_content=`$(get_temp) tmpl_objs_content_$$$$` ;	      \
	 $(UMASK) ;							      \
	 $(TOUCH) $$regular_objs_content $$template_objs_content ;	      \
	 objects="$(OBJECTS)" ;						      \
	 for i in $$objects ; do					      \
	     if [ -f $$i ] ; then					      \
	         $(ECHO) $(APEX_DIRECTORY)/$$i >> $$regular_objs_content ;    \
	     else							      \
	     	 $(ERR) "Unable to find $$i in $(APEX_DIRECTORY)" ;	      \
	     fi ;							      \
	 done ;								      \
	 subdirs="$(APEX_SUBDIRS)" ;					      \
	 for i in $$subdirs ; do					      \
	     if [ -d $$i ] ; then 					      \
	       (cd $$i ; 						      \
	        $(MAKE) CONTENT_QUERY_RESULT=$$regular_objs_content           \
			APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS=$(APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS) \
	                subdir_object_files_content.query ; 		      \
	       ) ;							      \
	     fi ;							      \
	 done ;								      \
	 $(CAT) $$regular_objs_content >> $(CONTENT_QUERY_RESULT) ;	      \
	 if [ "$(APEX_CPP_LIBRARY_INCLUDE_TEMPLATE_INSTANTIATIONS)" = true ] ; then \
	     if [ "$(VERBOSE)" = true ] ; then				      \
	         $(NOTE) "Computing template instantiations in $(APEX_VIEW)" ;\
	     fi ;							      \
	     if [ "$(APEX_PTREPOSITORY_DIRECTORY)" = "" ] ; then	      \
		ptr_dir="nil" ;						      \
	     else							      \
		ptr_dir="$(APEX_PTREPOSITORY_DIRECTORY)" ;		      \
	     fi ;							      \
	     CMD="$(tmpl_mgr) $(APEX_VIEW) query $$ptr_dir $$regular_objs_content $$template_objs_content $(CPP_OPTIONS)" ; \
	     if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;	      \
	     $(UMASK) ; 						      \
	     if $$CMD ; then						      \
		 : ; 							      \
	     else							      \
	         $(RM) $$regular_objs_content $$template_objs_content ;	      \
		 exit 1 ;						      \
	     fi ;							      \
	     $(CAT) $$template_objs_content >> $(CONTENT_QUERY_RESULT) ;      \
	 fi ; 								      \
	 $(RM) $$regular_objs_content $$template_objs_content

#######################################
# Standard Object Files Content Targets

object_files.object_files_content.query: view_object_files_content.query

library.object_files_content.query: view_object_files_content.query

layer_library.object_files_content.query: view_object_files_content.query

closure_library.object_files_content.query: view_object_files_content.query

mutual_library.object_files_content.query: view_object_files_content.query

mutual_member.object_files_content.query: view_object_files_content.query

external_library.object_files_content.query:

ptrepository.object_files_content.query:

###############################################################################
# Maintenance rules
#

# Perform maintenance of the build state.  Apex maintain always traverses to
# subdirectories within the view, so maintaining a directory does only the
# files within the directory.  These targets are run using the query makefile
# so that a missing dependency file doesn't prevent the operation.

.c.maintain .C.maintain .cc.maintain .cpp.maintain .cxx.maintain:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs maintaining: $< in $(APEX_DIRECTORY)" ;	      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 if [ "$(VERBOSE)" = true ] ; then				      \
	     $(NOTE) "Maintaining $< in $(APEX_DIRECTORY)" ;		      \
	 fi ;								      \
	 if [ -d $(DIR_MAKEFILES_PARENT) ] ; then			      \
	     if [ ! -f $(DIR_MAKEFILES_PARENT)/$*.dep ] ; then		      \
	         $(TOUCH) $(DIR_MAKEFILES_PARENT)/$*.dep ;		      \
	     fi ;							      \
	 fi ;								      \
	 CMD="$(maintain) $*.o" ;					      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(UMASK) ; $$CMD

maintain.all_files: maintain.all_files.announce
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Needs maintaining: $(APEX_DIRECTORY)" ;		      \
	     exit 0 ; 							      \
	 fi ; 								      \
	 CMD="$(maintain) -dir $(APEX_DIRECTORY)" ;			      \
	 if [ "$(VERBOSE)" = true ] ; then $(TRACE) $$CMD ; fi ;              \
	 $(UMASK) ; $$CMD

maintain.all_files.announce:
	@if [ "$(VERBOSE)" = true ] ; then				      \
	   $(NOTE) "Maintaining artifacts in $(APEX_DIRECTORY)";	      \
	 fi

maintain.dependency_files:
	@if [ "$(EFFORT_ONLY)" = true ] ; then 				      \
	     $(EFFORT) "Dependency files need maintaining: $(APEX_DIRECTORY)" ; \
	     exit 0 ; 							      \
	 fi ; 								      \
	 cd $(DIR_MAKEFILES_PARENT) ;					      \
	 $(UMASK) ;							      \
	 for d in `echo $(APEX_OBJECTS) $(APEX_MAIN_OBJECTS) | xargs -n1 | cut -d. -f1` ; do \
	     if [ ! -f $$d.dep ] ; then		      			      \
	         $(TOUCH) $$d.dep ;					      \
	     fi ;							      \
	 done


maintain.all_files_and_dependencies: maintain.dependency_files maintain.all_files

maintain.any_view.announce:
	@if [ "$(APEX_DIRECTORY_CLASS)" != "Directory" ] ; then		      \
	     $(NOTE) "Maintaining view $(APEX_VIEW)" ;      		      \
	 fi

maintain.any_view: maintain.any_view.announce maintain.all_files_and_dependencies

###########################
# Standard Maintain Targets

object_files.maintain: maintain.any_view

library.maintain: maintain.any_view

layer_library.maintain: maintain.any_view

closure_library.maintain: maintain.any_view

mutual_library.maintain: maintain.any_view

mutual_member.maintain: maintain.any_view

external_library.maintain:

ptrepository.maintain:

###############################################################################
###############################################################################
# Standard Targets for Apex Commands
#
# These are the targets that are generally used when an Apex build-oriented
# command is applied to a view or a subdirectory of a view.  Each build policy
# must supply a target for each of these.
#
# Each standard target references macros named CUSTOM_XXX_PRECONDITION and
# CUSTOM_XXX_ACTION which can be defined in the Customization.mk in order to
# customize the predefined operations.
# Each standard target also references the macro PRIMARY_XXX_TARGET
# which can be overridden in the the Customization.mk in order to change
# the usual primary affect of the standard targets.

prepared: check_build_params check_build_config $(CUSTOM_PREPARE_PRECONDITION) $(PRIMARY_PREPARE_TARGET)
	@: ; $(CUSTOM_PREPARE_ACTION)

parsed: check_build_params check_build_config $(CUSTOM_PARSE_PRECONDITION) $(PRIMARY_PARSE_TARGET)
	@: ; $(CUSTOM_PARSE_ACTION)

pretty_printed: check_build_params check_build_config $(CUSTOM_PRETTY_PRINT_PRECONDITION) $(PRIMARY_PRETTY_PRINT_TARGET)
	@: ; $(CUSTOM_PRETTY_PRINT_ACTION)

dependencies: check_build_params check_build_config $(CUSTOM_DEPENDENCIES_PRECONDITION) $(PRIMARY_DEPENDENCIES_TARGET)
	@: ; $(CUSTOM_DEPENDENCIES_ACTION)

analyzed: check_build_params check_build_config $(CUSTOM_ANALYZE_PRECONDITION) $(PRIMARY_ANALYZE_TARGET)
	@: ; $(CUSTOM_ANALYZE_ACTION)

coded: check_build_params check_build_config $(CUSTOM_CODE_PRECONDITION) $(PRIMARY_CODE_TARGET)
	@: ; $(CUSTOM_CODE_ACTION)

linked: check_build_params check_build_config $(CUSTOM_LINK_PRECONDITION) $(PRIMARY_LINK_TARGET)
	@: ; $(CUSTOM_LINK_ACTION)

generated: check_build_params check_build_config $(CUSTOM_GENERATE_PRECONDITION) $(PRIMARY_GENERATE_TARGET)
	@: ; $(CUSTOM_GENERATE_ACTION)

checked_rules: check_build_params check_build_config $(CUSTOM_CHECK_RULES_PRECONDITION) $(PRIMARY_CHECK_RULES_TARGET)
	@: ; $(CUSTOM_CHECK_RULES_ACTION)

tested: check_build_params check_build_config $(CUSTOM_TEST_PRECONDITION) $(PRIMARY_TEST_TARGET)
	@: ; $(CUSTOM_TEST_ACTION)

delivered: check_build_params check_build_config $(CUSTOM_DELIVER_PRECONDITION) $(PRIMARY_DELIVER_TARGET)
	@: ; $(CUSTOM_DELIVER_ACTION)

finalized: check_build_params check_build_config $(CUSTOM_FINALIZE_PRECONDITION) $(PRIMARY_FINALIZE_TARGET)
	@: ; $(CUSTOM_FINALIZE_ACTION)

built: check_build_params check_build_config $(CUSTOM_BUILD_PRECONDITION) $(PRIMARY_BUILD_TARGET)
	@: ; $(CUSTOM_BUILD_ACTION)

disassembled: check_build_params check_build_config $(CUSTOM_DISASSEMBLE_PRECONDITION) $(PRIMARY_DISASSEMBLE_TARGET)
	@: ; $(CUSTOM_DISASSEMBLE_ACTION)

cleaned: check_build_params check_build_config $(CUSTOM_CLEAN_PRECONDITION) $(PRIMARY_CLEAN_TARGET)
	@: ; $(CUSTOM_CLEAN_ACTION)

maintained: check_build_params check_build_config $(CUSTOM_MAINTAIN_PRECONDITION) $(PRIMARY_MAINTAIN_TARGET)
	@: ; $(CUSTOM_MAINTAIN_ACTION)


###############################################################################
# Check Build Paramters
#
# Checks that the BUILD_POLICY and the BUILD_COMPONENT are set to values
# that are supported in the makefiles.  The possible values are specified
# in Definitions.mk

check_build_params:
	@if [ "$(APEX_BUILD_POLICY)" = "" ] ; then			      \
	    $(ERR) "The BUILD_POLICY switch is not set in $(APEX_VIEW)" ;     \
	    exit 1 ;							      \
	 fi ;								      \
	 if [ "$(SUPPORTED_BUILD_POLICIES)" != "" ] ; then		      \
	     supported_policy=false ;					      \
	     for policy in $(SUPPORTED_BUILD_POLICIES) ; do		      \
	         if [ $$policy = "$(APEX_BUILD_POLICY)" ] ; then	      \
	             supported_policy=true ;				      \
	             break ;						      \
	         fi ;							      \
	     done ;							      \
	     if [ $$supported_policy = false ] ; then			      \
	         $(ERR) "The BUILD_POLICY value '$(APEX_BUILD_POLICY)' is not supported in $(APEX_VIEW)" ; \
	         $(ERR) "Supported policies are: $(SUPPORTED_BUILD_POLICIES)";\
	         exit 1 ;						      \
	     fi ;							      \
	 fi ;								      \
	 if [ "$(APEX_BUILD_COMPONENT)" = "" ] ; then			      \
	    $(ERR) "The BUILD_COMPONENT switch is not set in $(APEX_VIEW)" ;  \
	    exit 1 ;							      \
	 fi ;								      \
	 if [ "$(SUPPORTED_BUILD_COMPONENTS)" != "" ] ; then		      \
	     supported_component=false ;				      \
	     for component in $(SUPPORTED_BUILD_COMPONENTS) ; do	      \
	         if [ $$component = "$(APEX_BUILD_COMPONENT)" ] ; then	      \
	             supported_component=true ;				      \
	             break ;						      \
	         fi ;							      \
	     done ;							      \
	     if [ $$supported_component = false ] ; then		      \
	         $(ERR) "The BUILD_COMPONENT value '$(APEX_BUILD_COMPONENT)' is not supported in $(APEX_VIEW)" ; \
	         $(ERR) "Supported components are: $(SUPPORTED_BUILD_COMPONENTS)"; \
	         exit 1 ;						      \
	     fi ;							      \
	 fi

###############################################################################
###############################################################################
# Check Build Configuration
#
# HP-3 compiler with variant aCC does not need any template instantiation check

check_build_config:
	@if [ "$(APEX_CPP_COMPILER_KIND)" != "HP-3" -o                        \
	      "$(APEX_CPP_COMPILER_VARIANT)" != "aCC" ] ; then                \
	  if [ "$(VERBOSE)" = "true" ] ; then				      \
	     CMD="$(APEX_SHELL) $(check_build_config) -verbose $(APEX_VIEW)" ;\
	     $(TRACE) $$CMD ;						      \
	  else								      \
	     CMD="$(APEX_SHELL) $(check_build_config) -quiet $(APEX_VIEW)";   \
	  fi ;								      \
	  $$CMD ;                                                             \
	fi

###############################################################################
###############################################################################
# Primary Queries
#
# These are the targets that are used to query the makefile state for:
# 	1. The contribution to a link
#	2. The object files that will contribute to libraries.
#
# Each build policy must provide these targets.


link_contribution.query	: $(PRIMARY_LINK_CONTRIBUTION_QUERY_TARGET)

object_files_content.query: $(PRIMARY_OBJECT_FILES_CONTENT_QUERY_TARGET)

cpp_ptrepository.query: $(APEX_BUILD_POLICY).cpp_ptrepository.query

###############################################################################
# Standard Renames

all all_units: coded

all_programs: linked

$(APEX_LIBRARY_NAME): ALWAYS_OBSOLETE
	@( cd $(APEX_VIEW) ;						      \
	   $(MAKE) $(STANDARD_OPTION_SETTINGS) $(COMPILER_OPTION_SETTINGS) coded \
	 )

clean: cleaned

depend: dependencies

default: coded


###############################################################################
# Predefined Dependencies
#
# All object files are considered to be dependent on:
#     1. The Options makefile which contains the values of compilation options.
#     2. The Master makefile which defines the rules of compilation.
#     3. The Compilation configuration which is updated whenever imports
#        are changed.
#     4. The -force option.


$(APEX_S_OBJECTS) DUMMYS: $(OPTIONS_S_MAKEFILE)

$(APEX_C_OBJECTS) $(APEX_MAIN_C_OBJECTS) DUMMYC: $(OPTIONS_C_MAKEFILE)

$(APEX_CPP_OBJECTS) $(APEX_MAIN_CPP_OBJECTS) DUMMYCPP: $(OPTIONS_CPP_MAKEFILE)

$(APEX_OBJECTS) $(APEX_MAIN_OBJECTS) DUMMY3: $(OPTIONS_MAKEFILE)

$(APEX_OBJECTS) $(APEX_MAIN_OBJECTS) DUMMY4: $(MASTER_MAKEFILE) $(INCLUDES_MAKEFILE)

$(APEX_OBJECTS) $(APEX_MAIN_OBJECTS) DUMMY5: $(EXPLICITS_CFG) $(MUTUALS_CFG) $(EXPLICITS_EXPORT_SETS) $(COMPILATION_CFG) $(EXPORT_SETS)

$(APEX_OBJECTS) $(APEX_MAIN_OBJECTS) DUMMY6: $(FORCE)

$(APEX_JAVA_CLASSES)                 DUMMY7: ALWAYS_OBSOLETE
# The Java compiler manages the real dependencies.


###############################################################################
# End of Master Makefile
