/*+--------------------------------------------------------------------------+
 *| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
 *| This software contains proprietary and confidential information of       |
 *| Rational and its suppliers. Use, disclosure or reproduction is           |
 *| prohibited without the prior express written consent of Rational.        |
 *+--------------------------------------------------------------------------+
 */
#ifndef TMS_COMMUNICATION_INTERFACE_H
#define TMS_COMMUNICATION_INTERFACE_H
#include "testmate/tms_header_macros.h"
TMS_BEGIN_CPLUSPLUS

#include "testmate/tms.h"

/*
*  This interface to the TMS communications routines is supplied so
*  that Ada, C and C++ can share common communications logic.  With
*  out this interface hiding the support routines becomes much more
*  difficult.
*/


/* Test List termination reasons */
enum Tl_Termination_Reason {
    Tl_None = 0, 
    Tl_Elapsed_Time_Overflow, 
    Tl_Cpu_Time_Overflow,
    Tl_Max_Number_Of_Failures_Exceeded,
    Tl_Prior_Unhandled_Exception, 
    Tl_Internal_Error, 
    Tl_Other_Error
};

/* Test Case termination reasons */
enum Tc_Termination_Reason {
    Tc_None = 0, 
    Tc_Failed_Precondition, 
    Tc_Missing_Io_File, 
    Tc_Elapsed_Time_Overflow,
    Tc_Cpu_Time_Overflow, 
    Tc_Unhandled_Exception, 
    Tc_Internal_Error, 
    Tc_Other_Error
};


/* Test List relative important data is returned in the following construct */
struct Tl_Fields {
    int  Overall_Number_Of_Failures;
};

extern const struct Tl_Fields Nil_Tl_Fields;

/* Values returned from communication with TMS server */
struct Tc_Fields {
    char *Input;
    char *Output;
    char *Error_Output;
    char *Execution_Context;
    int  Allow_Coverage_Analysis;
    int  Cpu_Limit;
    int  Elapsed_Limit;
  /*Tl_Termination_Reason Tl_Term_Reason;*/
    int Tl_Term_Reason;
  /*Tc_Termination_Reason Tc_Term_Reason;*/
    int Tc_Term_Reason;
};

extern const struct Tc_Fields Nil_Tc_Fields;


/* Check if connected to server; connects if can and returns 0 if connects */
extern int Tms_Comm_Connected ();


/* Call back to server to resolve placeholders in parameter strings */
extern char *Tms_Comm_Resolve_Placeholders (char *In_The_String, char *Result);
extern char *Tms_Comm_Resolve_Placeholders_n (char *In_The_String, char *Result, int len);

extern char *Tms_Comm_Script_Resolve_Placeholders (char *In_The_String);


/* Synchronize with TMS server at script start and fetch TL specific values */
extern int Tms_Comm_Initialize (struct Tl_Fields *Tl_Info);


/* Synchronize with TMS server at Test Case start & fetch TC specific values */
extern int Tms_Comm_Start_Test_Case (struct Tc_Fields *Tc_Info);


/* Synchronize before start of the Test Driver for CA initialization purposes */
extern int Tms_Comm_Start_Driver ();


/* Synchronize after end of the Test Driver for CA termination purposes */
extern int Tms_Comm_End_Driver ();


/* Synchronize after end of the Test Case to stay sync'ed with TMS server */
extern int Tms_Comm_End_Test_Case (/*Tc_Termination_Reason*/ int The_Term_Reason);


/*
//  The following routines are TMS user support routines.  Just pass along
//  the appropriate message to the TMS server (if connected).  If not 
//  connected to the server, and can't connect, issue a warning and return
//  a default value.
*/

extern void Tms_Comm_Register_Result
		(char *The_Result, char *For_The_Test_Case);

extern char *Tms_Comm_Switch_Value (char *The_Switch, char *Result);
extern char *Tms_Comm_Switch_Value_n (char *The_Switch, char *Result, int len);

extern int Tms_Comm_Number_Of_Tests (char* With_Result);

extern char *Tms_Comm_Prior_Test_Case_Result (char *Result);

extern char *Tms_Comm_The_Test_Case_Result (char *Of_Test_Case, char *Result);

extern void Tms_Comm_Register_Environment ();

extern void Tms_Comm_Put_Line (char *Message, Tms_Log_Msg_Kind Msg_Kind);

extern void Tms_Comm_Assert (int Condition, char *Description );

extern void Tms_Comm_Assertion_Log_Control (Tms_Log_Mode Log_These_Entries);

extern Tms_Assertion_Status Tms_Comm_Check_Assertions ();

TMS_END_CPLUSPLUS
#endif

