/*+--------------------------------------------------------------------------+
 *| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
 *| This software contains proprietary and confidential information of       |
 *| Rational and its suppliers. Use, disclosure or reproduction is           |
 *| prohibited without the prior express written consent of Rational.        |
 *+--------------------------------------------------------------------------+
 */
/*                    tms_communications.h                     */

#ifndef TMS_COMMUNICATIONS_H
#define TMS_COMMUNICATIONS_H
#include "testmate/tms_header_macros.h"
TMS_BEGIN_CPLUSPLUS

/*
// This package is used as the conduit for communications between the
// test scripts (or separately executing jobs) and the execution server
// which processes the requests.  For requests that require a response
// the execution server will fill in a response and respond to the request.
// In reality, all requests are responded to so that the program stays
// synchronized with the associated server program.
//
// Implementation of the communications is performed through a socket
// allocated by the server (before execution of the script is started)
// The hostid:port_number pair of this socket is passed to the
// script via an environment variable (the variable is defined in the
// current process, and it is assumed that it is passed on to any
// child processes forked from this parent).  The
// script connects to the socket to complete the connection.  A test
// using TMS without the execution server running will encounter a void
// environment variable setting and simply ignore requests.
//
// The original version of this package contained a request record that
// was passed through the socket (in place of the Byte_Array used now).
// This implementation proved expensive in terms of the sizes of requests
// being passed.  Attempts to reduce the size of requests increased the
// number of discriminants associated with the request, which affected
// all requests instead of just those requiring the discriminants.  The
// lesser of the evils was deemed a separate encoding package that created
// simple byte arrays that are passed.  The down side is ofcourse the
// immediate argument checking that was handled by the request record
// now being done on both sides of the socket.  On the up side, this
// implementation is much more flexible and can easily be used for other
// uses in the future.
*/


/*
// The following routines maintain the socket used to communicate with the
// server.  The 'int' result of the first two operations flags whether the
// operation was successful or not.  The 'int' result of Tms_Is_Connected
// determines if the socket connection has been made.  A result of zero 
// implies an okay or true result.
*/
extern int Tms_Connect ();
extern int Tms_Disconnect ();
extern int Tms_Is_Connected ();


/*
// The following routine sends the specified request to the server.  The
// result is zero if successful, -1 if unsuccessful.
*/
extern int Tms_Send TMS_PROTO((char *Request, int Length));


/*
// The following routine waits for a response from the server and returns
// the still encoded response.  The result is placed at the location 
// pointed to by Response and the length is returned as the result.  A
// negative result implies a failure mode.
*/
extern int Tms_Receive TMS_PROTO((char *Response));

TMS_END_CPLUSPLUS

#endif
