/*+--------------------------------------------------------------------------+
 *| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
 *| This software contains proprietary and confidential information of       |
 *| Rational and its suppliers. Use, disclosure or reproduction is           |
 *| prohibited without the prior express written consent of Rational.        |
 *+--------------------------------------------------------------------------+
 */
/* 
//  TestMate Script Support routines ...
//
//  Only three routines are exported to support script execution.
//
//  Tms_Script_Support_Initialize:  Synchronizes post script elaboration
//       with the TMS server, waiting to start script execution.
//
//  Tms_Test_Case_Invoke:  Executes the various routines that make up a
//       Test Case through a series of call backs to the script.  The
//       termination of a test script is supported through a non-zero
//       result being returned by this routine.  All error reporting
//       is done by this routine, so that it is not necessary to do from
//       inside the script.
//
//  Tms_Resolve_Placeholders:  Calls back to the server to get nested
//       placeholders resolved from within parameters to user routines
//       called from within the test script.  Buffers resulting string
//       and deallocates when Tms_Test_Case_Invoke ends.
*/

#ifndef TMS_SCRIPT_SUPPORT_H
#define TMS_SCRIPT_SUPPORT_H

#include "testmate/tms_header_macros.h"
TMS_BEGIN_CPLUSPLUS


extern int  Tms_Script_Support_Initialize ();
extern int  Tms_Script_Support_Finialize ();

extern int  Tms_Test_Case_Invoke (int Test_Case_Id,
                                  void (*prolog) (int), /* = User_Test_Prolog */
                                  int  (*precond) (int),/* = User_Precond */
                                  void (*driver) (int), /* = Test_Driver */
                                  void (*eval) (int),   /* = User_Eval_Action */
                                  void (*epilog) (int));/* = User_Test_Epilog */

extern char *Tms_Script_Resolve_Placeholders (char *param);


TMS_END_CPLUSPLUS
#endif
